/*
    Copyright 2007, Joe Davidson <joedavidson@gmail.com>

    This file is part of FFTPatcher.

    FFTPatcher is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FFTPatcher is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FFTPatcher.  If not, see <http://www.gnu.org/licenses/>.
*/

using System.Collections.Generic;
using System.IO;
using System;
using PatcherLib.Utilities;
using PatcherLib.Datatypes;

namespace PatcherLib.Iso
{
    public static class PsxIso
    {
        #regionPublicProperties(19)

        public static KnownPosition Abilities { get; private set; }

        public static KnownPosition AbilityEffects { get; private set; }
        public static KnownPosition ReactionAbilityEffects { get; private set; }

        public static KnownPosition ActionEvents { get; private set; }

        public static KnownPosition ENTD1 { get; private set; }

        public static KnownPosition ENTD2 { get; private set; }

        public static KnownPosition ENTD3 { get; private set; }

        public static KnownPosition ENTD4 { get; private set; }

        public static KnownPosition InflictStatuses { get; private set; }

        public static KnownPosition JobLevels { get; private set; }

        public static KnownPosition Jobs { get; private set; }

        public static KnownPosition MonsterSkills { get; private set; }

        public static KnownPosition MoveFindItems { get; private set; }

        public static KnownPosition OldItemAttributes { get; private set; }

        public static KnownPosition OldItems { get; private set; }

        public static KnownPosition PoachProbabilities { get; private set; }

        public static KnownPosition SkillSets { get; private set; }

        public static KnownPosition StatusAttributes { get; private set; }

        public static KnownPosition StoreInventories { get; private set; }

        public static KnownPosition NumberOfSectorsBigEndian { get; private set; }
        public static KnownPosition NumberOfSectorsLittleEndian { get; private set; }


        public static KnownPosition AbilityAnimations { get; private set; }
        public static KnownPosition Propositions { get; private set; }
        #endregionPublicProperties

        #regionConstructors(1)

        static PsxIso()
        {
            Propositions = new KnownPosition( Sectors.WORLD_WLDCORE_BIN, 0x36380, 0xA7C );
            Abilities = new KnownPosition( Sectors.SCUS_942_21, 0x4F3F0, 9414 );
            AbilityEffects = new KnownPosition( Sectors.BATTLE_BIN, 0x14F3F0, 0x2E0 );
            ReactionAbilityEffects = new KnownPosition( Sectors.BATTLE_BIN, 0x014F73C, 0x40 );
            ActionEvents = new KnownPosition( Sectors.SCUS_942_21, 0x564B4, 224 );
            InflictStatuses = new KnownPosition( Sectors.SCUS_942_21, 0x547C4, 0x300 );
            Jobs = new KnownPosition( Sectors.SCUS_942_21, 0x518B8, 0x1E00 );
            JobLevels = new KnownPosition( Sectors.SCUS_942_21, 0x568C4, 0xD0 );
            MonsterSkills = new KnownPosition( Sectors.SCUS_942_21, 0x563C4, 0xF0 );
            OldItemAttributes = new KnownPosition( Sectors.SCUS_942_21, 0x54AC4, 0x7D0 );
            OldItems = new KnownPosition( Sectors.SCUS_942_21, 0x536B8, 0x110A );
            PoachProbabilities = new KnownPosition( Sectors.SCUS_942_21, 0x56864, 0x60 );
            StatusAttributes = new KnownPosition( Sectors.SCUS_942_21, 0x565E4, 0x280 );
            SkillSets = new KnownPosition( Sectors.SCUS_942_21, 0x55294, 0x1130 );
            ENTD1 = new KnownPosition( Sectors.BATTLE_ENTD1_ENT, 0, 81920 );
            ENTD2 = new KnownPosition( Sectors.BATTLE_ENTD2_ENT, 0, 81920 );
            ENTD3 = new KnownPosition( Sectors.BATTLE_ENTD3_ENT, 0, 81920 );
            ENTD4 = new KnownPosition( Sectors.BATTLE_ENTD4_ENT, 0, 81920 );
            MoveFindItems = new KnownPosition( Sectors.BATTLE_BIN, 0x8EE74, 0x800 );
            StoreInventories = new KnownPosition( Sectors.WORLD_WORLD_BIN, 0xAD844, 0x200 );
            NumberOfSectorsBigEndian = new KnownPosition( (Sectors)16, 0x54, 4 );
            NumberOfSectorsLittleEndian = new KnownPosition( (Sectors)16, 0x50, 4 );
            AbilityAnimations = new KnownPosition( Sectors.BATTLE_BIN, 0x2CE10, 0x600 );
        }

        #endregionConstructors

        #regionPublicMethods(3)

        public static byte[] GetBlock( Stream iso, KnownPosition knownPositions )
        {
            return ReadFile( iso, knownPositions.Sector, knownPositions.StartLocation, knownPositions.Length );
        }

        public static void PatchPsxIso( Stream iso, IEnumerable<PatcherLib.Datatypes.PatchedByteArray> patches )
        {
            foreach (var patch in patches)
            {
                PatchPsxIso( iso, patch );
            }
        }

        public static void PatchPsxIso( Stream iso, PatcherLib.Datatypes.PatchedByteArray patch )
        {
            if (patch is STRPatchedByteArray)
            {
                PatchPsxIso( iso, (STRPatchedByteArray)patch );
            }
            else
            {
                IsoPatch.PatchFileAtSector( IsoPatch.IsoType.Mode2Form1, iso, true, patch.Sector,
                    patch.Offset, patch.GetBytes(), true );
            }
        }

        public static void PatchPsxIso( Stream iso, PatcherLib.Datatypes.STRPatchedByteArray patch )
        {
            ReplaceStrFile( iso, (Sectors)patch.Sector, patch.GetBytes() );
        }

        public static byte[] ReadFile( Stream iso, Sectors file, int offset, int length )
        {
            return IsoPatch.ReadFile( IsoPatch.IsoType.Mode2Form1, iso, (int)file, offset, length );
        }

        public static byte[] ReadFile( Stream iso, KnownPosition pos )
        {
            return ReadFile( iso, pos.Sector, pos.StartLocation, pos.Length );
        }

        public static void ReplaceStrFile( Stream iso, Sectors file, IList<byte> bytes )
        {
            const int bytesPerSector = 2336;
            const int bytesPerSectorM2 = 2352;
            if (bytes.Count % bytesPerSector != 0)
            {
                throw new ArgumentException( string.Format( "new STR file length must be a multiple of {0}", bytesPerSector ) );
            }

            int numSectors = bytes.Count / bytesPerSector;

            byte[] tempSector = new byte[bytesPerSectorM2];

            int startSector = (int)file;

            for (int i = 0; i < numSectors; i++)
            {
                int outputSector = startSector + i;
                iso.Seek( outputSector * bytesPerSectorM2, SeekOrigin.Begin );

                // Keep the first 16 bytes from the original image
                // It contains  the 00FFFFFF FFFFFFFF FFFFFF00 and MM:SS:FF bytes
                iso.Read( tempSector, 0, (bytesPerSectorM2 - bytesPerSector) );

                // Copy the 2336 bytes from the input file to the buffer
                bytes.Sub( i * bytesPerSector, (i + 1) * bytesPerSector - 1 ).CopyTo( tempSector, (bytesPerSectorM2 - bytesPerSector) );

                // The output from MC32.EXE doesn't have these bytes set properly:
                tempSector[16] = 0x01;
                tempSector[20] = 0x01;

                if ((tempSector[18] & 0x4) == 0)
                {
                    // Video or Data
                    // NOT audio, so need to update the 280 byte ecc/edc block
                    IsoPatch.GenerateEccEdc( tempSector, IsoPatch.IsoType.Mode2Form1 );
                }
                else
                {
                    // CDXA M2F2
                    IsoPatch.GenerateEccEdc( tempSector, IsoPatch.IsoType.Mode2Form2 );
                }

                // Finished mucking with the sector, write it back to the ISO
                iso.Seek( outputSector * bytesPerSectorM2, SeekOrigin.Begin );
                iso.Write( tempSector, 0, bytesPerSectorM2 );
            }
        }

        #endregionPublicMethods

        public enum Sectors
        {
            BATTLE_10M_SPR = 59862,
            BATTLE_10W_SPR = 59881,
            BATTLE_20M_SPR = 59900,
            BATTLE_20W_SPR = 59919,
            BATTLE_40M_SPR = 59938,
            BATTLE_40W_SPR = 59957,
            BATTLE_60M_SPR = 59976,
            BATTLE_60W_SPR = 59995,
            BATTLE_ADORA_SPR = 57141,
            BATTLE_AGURI_SPR = 57164,
            BATTLE_AJORA_SPR = 57186,
            BATTLE_ARLI_SPR = 57208,
            BATTLE_ARLI2_SP2 = 60113,
            BATTLE_ARU_SPR = 57229,
            BATTLE_ARUFU_SPR = 57251,
            BATTLE_ARUMA_SPR = 57273,
            BATTLE_ARUTE_SEQ = 57062,
            BATTLE_ARUTE_SHP = 57034,
            BATTLE_ARUTE_SPR = 57295,
            BATTLE_BARITEN_SPR = 57319,
            BATTLE_BARU_SPR = 57341,
            BATTLE_BARUNA_SPR = 57363,
            BATTLE_BEHI_SPR = 57385,
            BATTLE_BEHI2_SP2 = 60129,
            BATTLE_BEIO_SPR = 57408,
            BATTLE_BIBU2_SP2 = 60145,
            BATTLE_BIBUROS_SPR = 57430,
            BATTLE_BOM_SPR = 57452,
            BATTLE_BOM2_SP2 = 60161,
            BATTLE_CLOUD_SPR = 57473,
            BATTLE_CYOKO_SEQ = 57053,
            BATTLE_CYOKO_SHP = 57026,
            BATTLE_CYOKO_SPR = 57494,
            BATTLE_CYOMON1_SPR = 60014,
            BATTLE_CYOMON2_SPR = 60033,
            BATTLE_CYOMON3_SPR = 60052,
            BATTLE_CYOMON4_SPR = 60071,
            BATTLE_DAISU_SPR = 57519,
            BATTLE_DAMI_SPR = 57541,
            BATTLE_DEMON_SPR = 57563,
            BATTLE_DEMON2_SP2 = 60177,
            BATTLE_DILY_SPR = 57586,
            BATTLE_DILY2_SPR = 57608,
            BATTLE_DILY3_SPR = 57630,
            BATTLE_DORA_SPR = 57652,
            BATTLE_DORA1_SPR = 57674,
            BATTLE_DORA2_SPR = 57697,
            BATTLE_DORA22_SP2 = 60193,
            BATTLE_EFC_FNT_TIM = 57000,
            BATTLE_EFF1_SEQ = 57080,
            BATTLE_EFF1_SHP = 57076,
            BATTLE_EFF2_SEQ = 57081,
            BATTLE_EFF2_SHP = 57078,
            BATTLE_ENTD1_ENT = 60353,
            BATTLE_ENTD2_ENT = 60393,
            BATTLE_ENTD3_ENT = 60433,
            BATTLE_ENTD4_ENT = 60473,
            BATTLE_ERU_SPR = 57720,
            BATTLE_FURAIA_SPR = 57742,
            BATTLE_FUSUI_M_SPR = 57761,
            BATTLE_FUSUI_W_SPR = 57783,
            BATTLE_FYUNE_SPR = 57805,
            BATTLE_GANDO_SPR = 57827,
            BATTLE_GARU_SPR = 57848,
            BATTLE_GIN_M_SPR = 57870,
            BATTLE_GOB_SPR = 57892,
            BATTLE_GORU_SPR = 57913,
            BATTLE_GYUMU_SPR = 57935,
            BATTLE_H61_SPR = 57957,
            BATTLE_H75_SPR = 57979,
            BATTLE_H76_SPR = 58001,
            BATTLE_H77_SPR = 58023,
            BATTLE_H78_SPR = 58045,
            BATTLE_H79_SPR = 58067,
            BATTLE_H80_SPR = 58089,
            BATTLE_H81_SPR = 58111,
            BATTLE_H82_SPR = 58133,
            BATTLE_H83_SPR = 58155,
            BATTLE_H85_SPR = 58177,
            BATTLE_HASYU_SPR = 58199,
            BATTLE_HEBI_SPR = 58223,
            BATTLE_HIME_SPR = 58247,
            BATTLE_HYOU_SPR = 58269,
            BATTLE_HYOU2_SP2 = 60209,
            BATTLE_IKA_SPR = 58291,
            BATTLE_IRON2_SP2 = 60225,
            BATTLE_IRON3_SP2 = 60241,
            BATTLE_IRON4_SP2 = 60257,
            BATTLE_IRON5_SP2 = 60273,
            BATTLE_ITEM_M_SPR = 58312,
            BATTLE_ITEM_W_SPR = 58334,
            BATTLE_KANBA_SPR = 58356,
            BATTLE_KANZEN_SEQ = 57064,
            BATTLE_KANZEN_SHP = 57035,
            BATTLE_KANZEN_SPR = 58378,
            BATTLE_KASANEK_SPR = 58402,
            BATTLE_KASANEM_SPR = 58422,
            BATTLE_KI_SPR = 58442,
            BATTLE_KNIGHT_M_SPR = 58465,
            BATTLE_KNIGHT_W_SPR = 58487,
            BATTLE_KURO_M_SPR = 58509,
            BATTLE_KURO_W_SPR = 58532,
            BATTLE_KYUKU_SPR = 58554,
            BATTLE_LEDY_SPR = 58578,
            BATTLE_MARA_SPR = 58600,
            BATTLE_MINA_M_SPR = 58621,
            BATTLE_MINA_W_SPR = 58643,
            BATTLE_MINOTA_SPR = 58665,
            BATTLE_MINOTA2_SP2 = 60289,
            BATTLE_MOL_SPR = 58689,
            BATTLE_MOL2_SP2 = 60305,
            BATTLE_MON_SEQ = 57055,
            BATTLE_MON_SHP = 57030,
            BATTLE_MONK_M_SPR = 58712,
            BATTLE_MONK_W_SPR = 58734,
            BATTLE_MONO_M_SPR = 58756,
            BATTLE_MONO_W_SPR = 58778,
            BATTLE_MUSU_SPR = 58800,
            BATTLE_NINJA_M_SPR = 58822,
            BATTLE_NINJA_W_SPR = 58844,
            BATTLE_ODORI_W_SPR = 58866,
            BATTLE_ONMYO_M_SPR = 58888,
            BATTLE_ONMYO_W_SPR = 58910,
            BATTLE_ORAN_SPR = 58932,
            BATTLE_ORU_SPR = 58954,
            BATTLE_OTHER_SEQ = 57058,
            BATTLE_OTHER_SHP = 57032,
            BATTLE_OTHER_SPR = 57124,
            BATTLE_RAFA_SPR = 58976,
            BATTLE_RAGU_SPR = 58998,
            BATTLE_RAMUZA_SPR = 59021,
            BATTLE_RAMUZA2_SPR = 59043,
            BATTLE_RAMUZA3_SPR = 59065,
            BATTLE_REZE_SPR = 59087,
            BATTLE_REZE_D_SPR = 59109,
            BATTLE_RUDO_SPR = 59132,
            BATTLE_RUKA_SEQ = 57060,
            BATTLE_RYU_M_SPR = 59154,
            BATTLE_RYU_W_SPR = 59176,
            BATTLE_SAMU_M_SPR = 59198,
            BATTLE_SAMU_W_SPR = 59220,
            BATTLE_SAN_M_SPR = 59242,
            BATTLE_SAN_W_SPR = 59264,
            BATTLE_SERIA_SPR = 59286,
            BATTLE_SIMON_SPR = 59308,
            BATTLE_SIRO_M_SPR = 59331,
            BATTLE_SIRO_W_SPR = 59353,
            BATTLE_SOURYO_SPR = 60090,
            BATTLE_SUKERU_SPR = 59377,
            BATTLE_SYOU_M_SPR = 59398,
            BATTLE_SYOU_W_SPR = 59421,
            BATTLE_TETSU_SPR = 59443,
            BATTLE_THIEF_M_SPR = 59466,
            BATTLE_THIEF_W_SPR = 59488,
            BATTLE_TOKI_M_SPR = 59510,
            BATTLE_TOKI_W_SPR = 59532,
            BATTLE_TORI_SPR = 59554,
            BATTLE_TORI2_SP2 = 60321,
            BATTLE_TYPE1_SEQ = 57037,
            BATTLE_TYPE1_SHP = 57017,
            BATTLE_TYPE2_SEQ = 57041,
            BATTLE_TYPE2_SHP = 57021,
            BATTLE_TYPE3_SEQ = 57045,
            BATTLE_TYPE4_SEQ = 57049,
            BATTLE_URI_SPR = 59576,
            BATTLE_URI2_SP2 = 60337,
            BATTLE_VERI_SPR = 59596,
            BATTLE_VORU_SPR = 59619,
            BATTLE_WAJU_M_SPR = 59641,
            BATTLE_WAJU_W_SPR = 59663,
            BATTLE_WEP_SPR = 57082,
            BATTLE_WEP1_SEQ = 57072,
            BATTLE_WEP1_SHP = 57066,
            BATTLE_WEP2_SEQ = 57074,
            BATTLE_WEP2_SHP = 57069,
            BATTLE_WIGU_SPR = 59685,
            BATTLE_YUMI_M_SPR = 59707,
            BATTLE_YUMI_W_SPR = 59729,
            BATTLE_YUREI_SPR = 59751,
            BATTLE_ZARU_SPR = 59772,
            BATTLE_ZARU2_SPR = 59794,
            BATTLE_ZARUE_SPR = 59816,
            BATTLE_ZARUMOU_SPR = 59839,
            BATTLE_ZODIAC_BIN = 60513,
            EFFECT_E000_BIN = 61000,
            EFFECT_E001_BIN = 61026,
            EFFECT_E002_BIN = 61048,
            EFFECT_E003_BIN = 61071,
            EFFECT_E004_BIN = 61094,
            EFFECT_E005_BIN = 61140,
            EFFECT_E006_BIN = 61167,
            EFFECT_E007_BIN = 61194,
            EFFECT_E008_BIN = 61221,
            EFFECT_E009_BIN = 61244,
            EFFECT_E010_BIN = 61268,
            EFFECT_E011_BIN = 61292,
            EFFECT_E012_BIN = 61314,
            EFFECT_E013_BIN = 61337,
            EFFECT_E014_BIN = 61360,
            EFFECT_E015_BIN = 61383,
            EFFECT_E016_BIN = 61430,
            EFFECT_E017_BIN = 61445,
            EFFECT_E018_BIN = 61463,
            EFFECT_E019_BIN = 61487,
            EFFECT_E020_BIN = 61513,
            EFFECT_E021_BIN = 61536,
            EFFECT_E022_BIN = 61562,
            EFFECT_E023_BIN = 61585,
            EFFECT_E024_BIN = 61626,
            EFFECT_E025_BIN = 61649,
            EFFECT_E026_BIN = 61674,
            EFFECT_E027_BIN = 61699,
            EFFECT_E028_BIN = 61741,
            EFFECT_E029_BIN = 61765,
            EFFECT_E030_BIN = 61787,
            EFFECT_E031_BIN = 61809,
            EFFECT_E032_BIN = 61832,
            EFFECT_E033_BIN = 61855,
            EFFECT_E034_BIN = 61899,
            EFFECT_E035_BIN = 61922,
            EFFECT_E036_BIN = 61966,
            EFFECT_E037_BIN = 61989,
            EFFECT_E038_BIN = 61990,
            EFFECT_E039_BIN = 61991,
            EFFECT_E040_BIN = 62014,
            EFFECT_E041_BIN = 62026,
            EFFECT_E042_BIN = 62052,
            EFFECT_E043_BIN = 62053,
            EFFECT_E044_BIN = 62077,
            EFFECT_E045_BIN = 62090,
            EFFECT_E046_BIN = 62113,
            EFFECT_E047_BIN = 62136,
            EFFECT_E048_BIN = 62177,
            EFFECT_E049_BIN = 62178,
            EFFECT_E050_BIN = 62200,
            EFFECT_E051_BIN = 62222,
            EFFECT_E052_BIN = 62244,
            EFFECT_E053_BIN = 62267,
            EFFECT_E054_BIN = 62290,
            EFFECT_E055_BIN = 62312,
            EFFECT_E056_BIN = 62334,
            EFFECT_E057_BIN = 62356,
            EFFECT_E058_BIN = 62378,
            EFFECT_E059_BIN = 62401,
            EFFECT_E060_BIN = 62424,
            EFFECT_E061_BIN = 62439,
            EFFECT_E062_BIN = 62462,
            EFFECT_E063_BIN = 62485,
            EFFECT_E064_BIN = 62508,
            EFFECT_E065_BIN = 62509,
            EFFECT_E066_BIN = 62558,
            EFFECT_E067_BIN = 62599,
            EFFECT_E068_BIN = 62643,
            EFFECT_E069_BIN = 62686,
            EFFECT_E070_BIN = 62713,
            EFFECT_E071_BIN = 62764,
            EFFECT_E072_BIN = 62809,
            EFFECT_E073_BIN = 62850,
            EFFECT_E074_BIN = 62895,
            EFFECT_E075_BIN = 62942,
            EFFECT_E076_BIN = 62943,
            EFFECT_E077_BIN = 62987,
            EFFECT_E078_BIN = 63037,
            EFFECT_E079_BIN = 63078,
            EFFECT_E080_BIN = 63124,
            EFFECT_E081_BIN = 63170,
            EFFECT_E082_BIN = 63192,
            EFFECT_E083_BIN = 63214,
            EFFECT_E084_BIN = 63236,
            EFFECT_E085_BIN = 63258,
            EFFECT_E086_BIN = 63280,
            EFFECT_E087_BIN = 63302,
            EFFECT_E088_BIN = 63324,
            EFFECT_E089_BIN = 63350,
            EFFECT_E090_BIN = 63376,
            EFFECT_E091_BIN = 63402,
            EFFECT_E092_BIN = 63428,
            EFFECT_E093_BIN = 63454,
            EFFECT_E094_BIN = 63481,
            EFFECT_E095_BIN = 63508,
            EFFECT_E096_BIN = 63530,
            EFFECT_E097_BIN = 63552,
            EFFECT_E098_BIN = 63574,
            EFFECT_E099_BIN = 63598,
            EFFECT_E100_BIN = 63620,
            EFFECT_E101_BIN = 63642,
            EFFECT_E102_BIN = 63664,
            EFFECT_E103_BIN = 63686,
            EFFECT_E104_BIN = 63708,
            EFFECT_E105_BIN = 63730,
            EFFECT_E106_BIN = 63752,
            EFFECT_E107_BIN = 63774,
            EFFECT_E108_BIN = 63796,
            EFFECT_E109_BIN = 63818,
            EFFECT_E110_BIN = 63840,
            EFFECT_E111_BIN = 63862,
            EFFECT_E112_BIN = 63885,
            EFFECT_E113_BIN = 63908,
            EFFECT_E114_BIN = 63931,
            EFFECT_E115_BIN = 63954,
            EFFECT_E116_BIN = 63977,
            EFFECT_E117_BIN = 64000,
            EFFECT_E118_BIN = 64023,
            EFFECT_E119_BIN = 64046,
            EFFECT_E120_BIN = 64069,
            EFFECT_E121_BIN = 64092,
            EFFECT_E122_BIN = 64115,
            EFFECT_E123_BIN = 64137,
            EFFECT_E124_BIN = 64160,
            EFFECT_E125_BIN = 64183,
            EFFECT_E126_BIN = 64206,
            EFFECT_E127_BIN = 64229,
            EFFECT_E128_BIN = 64251,
            EFFECT_E129_BIN = 64274,
            EFFECT_E130_BIN = 64297,
            EFFECT_E131_BIN = 64319,
            EFFECT_E132_BIN = 64341,
            EFFECT_E133_BIN = 64363,
            EFFECT_E134_BIN = 64386,
            EFFECT_E135_BIN = 64416,
            EFFECT_E136_BIN = 64439,
            EFFECT_E137_BIN = 64463,
            EFFECT_E138_BIN = 64487,
            EFFECT_E139_BIN = 64513,
            EFFECT_E140_BIN = 64537,
            EFFECT_E141_BIN = 64561,
            EFFECT_E142_BIN = 64584,
            EFFECT_E143_BIN = 64626,
            EFFECT_E144_BIN = 64627,
            EFFECT_E145_BIN = 64628,
            EFFECT_E146_BIN = 64629,
            EFFECT_E147_BIN = 64630,
            EFFECT_E148_BIN = 64631,
            EFFECT_E149_BIN = 64632,
            EFFECT_E150_BIN = 64633,
            EFFECT_E151_BIN = 64634,
            EFFECT_E152_BIN = 64635,
            EFFECT_E153_BIN = 64636,
            EFFECT_E154_BIN = 64637,
            EFFECT_E155_BIN = 64651,
            EFFECT_E156_BIN = 64652,
            EFFECT_E157_BIN = 64674,
            EFFECT_E158_BIN = 64697,
            EFFECT_E159_BIN = 64720,
            EFFECT_E160_BIN = 64742,
            EFFECT_E161_BIN = 64767,
            EFFECT_E162_BIN = 64793,
            EFFECT_E163_BIN = 64794,
            EFFECT_E164_BIN = 64821,
            EFFECT_E165_BIN = 64846,
            EFFECT_E166_BIN = 64869,
            EFFECT_E167_BIN = 64894,
            EFFECT_E168_BIN = 64919,
            EFFECT_E169_BIN = 64943,
            EFFECT_E170_BIN = 64967,
            EFFECT_E171_BIN = 64991,
            EFFECT_E172_BIN = 65015,
            EFFECT_E173_BIN = 65038,
            EFFECT_E174_BIN = 65062,
            EFFECT_E175_BIN = 65109,
            EFFECT_E176_BIN = 65124,
            EFFECT_E177_BIN = 65167,
            EFFECT_E178_BIN = 65190,
            EFFECT_E179_BIN = 65213,
            EFFECT_E180_BIN = 65236,
            EFFECT_E181_BIN = 65259,
            EFFECT_E182_BIN = 65282,
            EFFECT_E183_BIN = 65305,
            EFFECT_E184_BIN = 65328,
            EFFECT_E185_BIN = 65351,
            EFFECT_E186_BIN = 65374,
            EFFECT_E187_BIN = 65397,
            EFFECT_E188_BIN = 65420,
            EFFECT_E189_BIN = 65443,
            EFFECT_E190_BIN = 65465,
            EFFECT_E191_BIN = 65487,
            EFFECT_E192_BIN = 65509,
            EFFECT_E193_BIN = 65533,
            EFFECT_E194_BIN = 65558,
            EFFECT_E195_BIN = 65580,
            EFFECT_E196_BIN = 65602,
            EFFECT_E197_BIN = 65624,
            EFFECT_E198_BIN = 65646,
            EFFECT_E199_BIN = 65668,
            EFFECT_E200_BIN = 65690,
            EFFECT_E201_BIN = 65712,
            EFFECT_E202_BIN = 65734,
            EFFECT_E203_BIN = 65756,
            EFFECT_E204_BIN = 65778,
            EFFECT_E205_BIN = 65802,
            EFFECT_E206_BIN = 65826,
            EFFECT_E207_BIN = 65850,
            EFFECT_E208_BIN = 65851,
            EFFECT_E209_BIN = 65875,
            EFFECT_E210_BIN = 65876,
            EFFECT_E211_BIN = 65891,
            EFFECT_E212_BIN = 65913,
            EFFECT_E213_BIN = 65936,
            EFFECT_E214_BIN = 65959,
            EFFECT_E215_BIN = 65982,
            EFFECT_E216_BIN = 66004,
            EFFECT_E217_BIN = 66026,
            EFFECT_E218_BIN = 66048,
            EFFECT_E219_BIN = 66070,
            EFFECT_E220_BIN = 66092,
            EFFECT_E221_BIN = 66114,
            EFFECT_E222_BIN = 66136,
            EFFECT_E223_BIN = 66158,
            EFFECT_E224_BIN = 66180,
            EFFECT_E225_BIN = 66202,
            EFFECT_E226_BIN = 66224,
            EFFECT_E227_BIN = 66246,
            EFFECT_E228_BIN = 66247,
            EFFECT_E229_BIN = 66287,
            EFFECT_E230_BIN = 66330,
            EFFECT_E231_BIN = 66373,
            EFFECT_E232_BIN = 66397,
            EFFECT_E233_BIN = 66421,
            EFFECT_E234_BIN = 66463,
            EFFECT_E235_BIN = 66485,
            EFFECT_E236_BIN = 66528,
            EFFECT_E237_BIN = 66569,
            EFFECT_E238_BIN = 66584,
            EFFECT_E239_BIN = 66599,
            EFFECT_E240_BIN = 66640,
            EFFECT_E241_BIN = 66684,
            EFFECT_E242_BIN = 66730,
            EFFECT_E243_BIN = 66782,
            EFFECT_E244_BIN = 66804,
            EFFECT_E245_BIN = 66847,
            EFFECT_E246_BIN = 66874,
            EFFECT_E247_BIN = 66917,
            EFFECT_E248_BIN = 66943,
            EFFECT_E249_BIN = 66972,
            EFFECT_E250_BIN = 66998,
            EFFECT_E251_BIN = 66999,
            EFFECT_E252_BIN = 67000,
            EFFECT_E253_BIN = 67001,
            EFFECT_E254_BIN = 67002,
            EFFECT_E255_BIN = 67003,
            EFFECT_E256_BIN = 67029,
            EFFECT_E257_BIN = 67061,
            EFFECT_E258_BIN = 67087,
            EFFECT_E259_BIN = 67133,
            EFFECT_E260_BIN = 67179,
            EFFECT_E261_BIN = 67201,
            EFFECT_E262_BIN = 67224,
            EFFECT_E263_BIN = 67247,
            EFFECT_E264_BIN = 67270,
            EFFECT_E265_BIN = 67293,
            EFFECT_E266_BIN = 67315,
            EFFECT_E267_BIN = 67338,
            EFFECT_E268_BIN = 67361,
            EFFECT_E269_BIN = 67384,
            EFFECT_E270_BIN = 67406,
            EFFECT_E271_BIN = 67429,
            EFFECT_E272_BIN = 67451,
            EFFECT_E273_BIN = 67474,
            EFFECT_E274_BIN = 67498,
            EFFECT_E275_BIN = 67499,
            EFFECT_E276_BIN = 67500,
            EFFECT_E277_BIN = 67501,
            EFFECT_E278_BIN = 67502,
            EFFECT_E279_BIN = 67503,
            EFFECT_E280_BIN = 67504,
            EFFECT_E281_BIN = 67505,
            EFFECT_E282_BIN = 67506,
            EFFECT_E283_BIN = 67507,
            EFFECT_E284_BIN = 67508,
            EFFECT_E285_BIN = 67509,
            EFFECT_E286_BIN = 67510,
            EFFECT_E287_BIN = 67511,
            EFFECT_E288_BIN = 67512,
            EFFECT_E289_BIN = 67552,
            EFFECT_E290_BIN = 67592,
            EFFECT_E291_BIN = 67632,
            EFFECT_E292_BIN = 67673,
            EFFECT_E293_BIN = 67714,
            EFFECT_E294_BIN = 67754,
            EFFECT_E295_BIN = 67794,
            EFFECT_E296_BIN = 67834,
            EFFECT_E297_BIN = 67835,
            EFFECT_E298_BIN = 67836,
            EFFECT_E299_BIN = 67837,
            EFFECT_E300_BIN = 67838,
            EFFECT_E301_BIN = 67839,
            EFFECT_E302_BIN = 67840,
            EFFECT_E303_BIN = 67841,
            EFFECT_E304_BIN = 67842,
            EFFECT_E305_BIN = 67843,
            EFFECT_E306_BIN = 67844,
            EFFECT_E307_BIN = 67867,
            EFFECT_E308_BIN = 67882,
            EFFECT_E309_BIN = 67896,
            EFFECT_E310_BIN = 67911,
            EFFECT_E311_BIN = 67934,
            EFFECT_E312_BIN = 67957,
            EFFECT_E313_BIN = 67980,
            EFFECT_E314_BIN = 68003,
            EFFECT_E315_BIN = 68027,
            EFFECT_E316_BIN = 68028,
            EFFECT_E317_BIN = 68042,
            EFFECT_E318_BIN = 68064,
            EFFECT_E319_BIN = 68086,
            EFFECT_E320_BIN = 68109,
            EFFECT_E321_BIN = 68131,
            EFFECT_E322_BIN = 68145,
            EFFECT_E323_BIN = 68159,
            EFFECT_E324_BIN = 68173,
            EFFECT_E325_BIN = 68188,
            EFFECT_E326_BIN = 68203,
            EFFECT_E327_BIN = 68217,
            EFFECT_E328_BIN = 68232,
            EFFECT_E329_BIN = 68247,
            EFFECT_E330_BIN = 68262,
            EFFECT_E331_BIN = 68276,
            EFFECT_E332_BIN = 68291,
            EFFECT_E333_BIN = 68305,
            EFFECT_E334_BIN = 68321,
            EFFECT_E335_BIN = 68335,
            EFFECT_E336_BIN = 68349,
            EFFECT_E337_BIN = 68367,
            EFFECT_E338_BIN = 68381,
            EFFECT_E339_BIN = 68398,
            EFFECT_E340_BIN = 68413,
            EFFECT_E341_BIN = 68436,
            EFFECT_E342_BIN = 68453,
            EFFECT_E343_BIN = 68468,
            EFFECT_E344_BIN = 68483,
            EFFECT_E345_BIN = 68499,
            EFFECT_E346_BIN = 68515,
            EFFECT_E347_BIN = 68530,
            EFFECT_E348_BIN = 68544,
            EFFECT_E349_BIN = 68559,
            EFFECT_E350_BIN = 68574,
            EFFECT_E351_BIN = 68589,
            EFFECT_E352_BIN = 68603,
            EFFECT_E353_BIN = 68619,
            EFFECT_E354_BIN = 68634,
            EFFECT_E355_BIN = 68657,
            EFFECT_E356_BIN = 68684,
            EFFECT_E357_BIN = 68694,
            EFFECT_E358_BIN = 68708,
            EFFECT_E359_BIN = 68722,
            EFFECT_E360_BIN = 68737,
            EFFECT_E361_BIN = 68751,
            EFFECT_E362_BIN = 68766,
            EFFECT_E363_BIN = 68780,
            EFFECT_E364_BIN = 68795,
            EFFECT_E365_BIN = 68809,
            EFFECT_E366_BIN = 68833,
            EFFECT_E367_BIN = 68856,
            EFFECT_E368_BIN = 68879,
            EFFECT_E369_BIN = 68902,
            EFFECT_E370_BIN = 68925,
            EFFECT_E371_BIN = 68948,
            EFFECT_E372_BIN = 68978,
            EFFECT_E373_BIN = 69002,
            EFFECT_E374_BIN = 69032,
            EFFECT_E375_BIN = 69046,
            EFFECT_E376_BIN = 69060,
            EFFECT_E377_BIN = 69078,
            EFFECT_E378_BIN = 69093,
            EFFECT_E379_BIN = 69107,
            EFFECT_E380_BIN = 69121,
            EFFECT_E381_BIN = 69136,
            EFFECT_E382_BIN = 69155,
            EFFECT_E383_BIN = 69174,
            EFFECT_E384_BIN = 69198,
            EFFECT_E385_BIN = 69246,
            EFFECT_E386_BIN = 69288,
            EFFECT_E387_BIN = 69302,
            EFFECT_E388_BIN = 69316,
            EFFECT_E389_BIN = 69331,
            EFFECT_E390_BIN = 69346,
            EFFECT_E391_BIN = 69361,
            EFFECT_E392_BIN = 69375,
            EFFECT_E393_BIN = 69390,
            EFFECT_E394_BIN = 69413,
            EFFECT_E395_BIN = 69428,
            EFFECT_E396_BIN = 69446,
            EFFECT_E397_BIN = 69486,
            EFFECT_E398_BIN = 69527,
            EFFECT_E399_BIN = 69550,
            EFFECT_E400_BIN = 69565,
            EFFECT_E401_BIN = 69606,
            EFFECT_E402_BIN = 69630,
            EFFECT_E403_BIN = 69657,
            EFFECT_E404_BIN = 69702,
            EFFECT_E405_BIN = 69743,
            EFFECT_E406_BIN = 69784,
            EFFECT_E407_BIN = 69825,
            EFFECT_E408_BIN = 69840,
            EFFECT_E409_BIN = 69886,
            EFFECT_E410_BIN = 69919,
            EFFECT_E411_BIN = 69942,
            EFFECT_E412_BIN = 69967,
            EFFECT_E413_BIN = 69995,
            EFFECT_E414_BIN = 70017,
            EFFECT_E415_BIN = 70018,
            EFFECT_E416_BIN = 70019,
            EFFECT_E417_BIN = 70020,
            EFFECT_E418_BIN = 70021,
            EFFECT_E419_BIN = 70022,
            EFFECT_E420_BIN = 70023,
            EFFECT_E421_BIN = 70024,
            EFFECT_E422_BIN = 70025,
            EFFECT_E423_BIN = 70026,
            EFFECT_E424_BIN = 70027,
            EFFECT_E425_BIN = 70028,
            EFFECT_E426_BIN = 70029,
            EFFECT_E427_BIN = 70030,
            EFFECT_E428_BIN = 70031,
            EFFECT_E429_BIN = 70032,
            EFFECT_E430_BIN = 70033,
            EFFECT_E431_BIN = 70034,
            EFFECT_E432_BIN = 70035,
            EFFECT_E433_BIN = 70036,
            EFFECT_E434_BIN = 70037,
            EFFECT_E435_BIN = 70038,
            EFFECT_E436_BIN = 70039,
            EFFECT_E437_BIN = 70040,
            EFFECT_E438_BIN = 70041,
            EFFECT_E439_BIN = 70042,
            EFFECT_E440_BIN = 70043,
            EFFECT_E441_BIN = 70044,
            EFFECT_E442_BIN = 70045,
            EFFECT_E443_BIN = 70046,
            EFFECT_E444_BIN = 70047,
            EFFECT_E445_BIN = 70048,
            EFFECT_E446_BIN = 70049,
            EFFECT_E447_BIN = 70050,
            EFFECT_E448_BIN = 70051,
            EFFECT_E449_BIN = 70052,
            EFFECT_E450_BIN = 70053,
            EFFECT_E451_BIN = 70083,
            EFFECT_E452_BIN = 70114,
            EFFECT_E453_BIN = 70145,
            EFFECT_E454_BIN = 70179,
            EFFECT_E455_BIN = 70209,
            EFFECT_E456_BIN = 70234,
            EFFECT_E457_BIN = 70260,
            EFFECT_E458_BIN = 70284,
            EFFECT_E459_BIN = 70309,
            EFFECT_E460_BIN = 70331,
            EFFECT_E461_BIN = 70356,
            EFFECT_E462_BIN = 70391,
            EFFECT_E463_BIN = 70426,
            EFFECT_E464_BIN = 70461,
            EFFECT_E465_BIN = 70491,
            EFFECT_E466_BIN = 70526,
            EFFECT_E467_BIN = 70561,
            EFFECT_E468_BIN = 70587,
            EFFECT_E469_BIN = 70614,
            EFFECT_E470_BIN = 70640,
            EFFECT_E471_BIN = 70663,
            EFFECT_E472_BIN = 70689,
            EFFECT_E473_BIN = 70716,
            EFFECT_E474_BIN = 70742,
            EFFECT_E475_BIN = 70773,
            EFFECT_E476_BIN = 70803,
            EFFECT_E477_BIN = 70833,
            EFFECT_E478_BIN = 70864,
            EFFECT_E479_BIN = 70895,
            EFFECT_E480_BIN = 70919,
            EFFECT_E481_BIN = 70949,
            EFFECT_E482_BIN = 70965,
            EFFECT_E483_BIN = 70981,
            EFFECT_E484_BIN = 71006,
            EFFECT_E485_BIN = 71030,
            EFFECT_E486_BIN = 71061,
            EFFECT_E487_BIN = 71084,
            EFFECT_E488_BIN = 71085,
            EFFECT_E489_BIN = 71086,
            EFFECT_E490_BIN = 71087,
            EFFECT_E491_BIN = 71088,
            EFFECT_E492_BIN = 71089,
            EFFECT_E493_BIN = 71090,
            EFFECT_E494_BIN = 71091,
            EFFECT_E495_BIN = 71092,
            EFFECT_E496_BIN = 71093,
            EFFECT_E497_BIN = 71094,
            EFFECT_E498_BIN = 71095,
            EFFECT_E499_BIN = 71096,
            EFFECT_E500_BIN = 71097,
            EFFECT_E501_BIN = 71098,
            EFFECT_E502_BIN = 71099,
            EFFECT_E503_BIN = 71100,
            EFFECT_E504_BIN = 71101,
            EFFECT_E505_BIN = 71102,
            EFFECT_E506_BIN = 71103,
            EFFECT_E507_BIN = 71104,
            EFFECT_E508_BIN = 71105,
            EFFECT_E509_BIN = 71106,
            EFFECT_E510_BIN = 71128,
            EFFECT_E511_BIN = 71150,
            EVENT_ATCHELP_LZW = 6714,
            EVENT_ATTACK_OUT = 2448,
            EVENT_BONUS_BIN = 5824,
            EVENT_BTLEVT_BIN = 5771,
            EVENT_BUNIT_OUT = 2832,
            EVENT_CARD_OUT = 2768,
            EVENT_CHAPTER1_BIN = 5776,
            EVENT_CHAPTER2_BIN = 5780,
            EVENT_CHAPTER3_BIN = 5784,
            EVENT_CHAPTER4_BIN = 5788,
            EVENT_DEBUGCHR_OUT = 2512,
            EVENT_DEBUGMAP_OUT = 2064,
            EVENT_END1_BIN = 6394,
            EVENT_END2_BIN = 6458,
            EVENT_END3_BIN = 6522,
            EVENT_END4_BIN = 6586,
            EVENT_END5_BIN = 6650,
            EVENT_EQUIP_OUT = 2640,
            EVENT_ETC_OUT = 2576,
            EVENT_EVTCHR_BIN = 7500,
            EVENT_EVTFACE_BIN = 5707,
            EVENT_EVTOOL_OUT = 2960,
            EVENT_FONT_BIN = 3650,
            EVENT_FRAME_BIN = 3688,
            EVENT_GAMEOVER_BIN = 5792,
            EVENT_HELP_LZW = 7320,
            EVENT_HELPMENU_OUT = 2256,
            EVENT_ITEM_BIN = 6297,
            EVENT_JOBSTTS_OUT = 2384,
            EVENT_JOIN_LZW = 7256,
            EVENT_MAPTITLE_BIN = 3500,
            EVENT_OPEN_LZW = 7192,
            EVENT_OPTION_OUT = 2128,
            EVENT_REQUIRE_OUT = 2192,
            EVENT_SAMPLE_LZW = 7064,
            EVENT_SMALL_OUT = 2000,
            EVENT_SPELL_MES = 7000,
            EVENT_TEST_EVT = 3707,
            EVENT_UNIT_BIN = 5739,
            EVENT_WIN001_BIN = 6292,
            EVENT_WLDFACE_BIN = 6330,
            EVENT_WLDFACE4_BIN = 6314,
            EVENT_WLDHELP_LZW = 7416,
            EVENT_WORLD_LZW = 7128,
            MAP_MAP000_5 = 10000,
            MAP_MAP000_GNS = 10026,
            MAP_MAP001_11 = 10110,
            MAP_MAP001_14 = 10113,
            MAP_MAP001_15 = 10177,
            MAP_MAP001_18 = 10178,
            MAP_MAP001_19 = 10242,
            MAP_MAP001_22 = 10243,
            MAP_MAP001_23 = 10307,
            MAP_MAP001_26 = 10308,
            MAP_MAP001_27 = 10372,
            MAP_MAP001_30 = 10373,
            MAP_MAP001_31 = 10437,
            MAP_MAP001_34 = 10438,
            MAP_MAP001_35 = 10502,
            MAP_MAP001_38 = 10503,
            MAP_MAP001_39 = 10567,
            MAP_MAP001_42 = 10568,
            MAP_MAP001_43 = 10632,
            MAP_MAP001_46 = 10633,
            MAP_MAP001_47 = 10697,
            MAP_MAP001_48 = 10698,
            MAP_MAP001_52 = 10701,
            MAP_MAP001_53 = 10765,
            MAP_MAP001_57 = 10768,
            MAP_MAP001_58 = 10832,
            MAP_MAP001_62 = 10835,
            MAP_MAP001_63 = 10899,
            MAP_MAP001_67 = 10902,
            MAP_MAP001_68 = 10966,
            MAP_MAP001_72 = 10969,
            MAP_MAP001_73 = 11033,
            MAP_MAP001_77 = 11036,
            MAP_MAP001_78 = 11100,
            MAP_MAP001_8 = 10027,
            MAP_MAP001_82 = 11103,
            MAP_MAP001_83 = 11167,
            MAP_MAP001_87 = 11170,
            MAP_MAP001_88 = 11234,
            MAP_MAP001_9 = 10091,
            MAP_MAP001_92 = 11237,
            MAP_MAP001_93 = 11301,
            MAP_MAP001_GNS = 11304,
            MAP_MAP002_12 = 11396,
            MAP_MAP002_13 = 11460,
            MAP_MAP002_16 = 11461,
            MAP_MAP002_17 = 11525,
            MAP_MAP002_20 = 11526,
            MAP_MAP002_21 = 11590,
            MAP_MAP002_24 = 11591,
            MAP_MAP002_25 = 11655,
            MAP_MAP002_28 = 11656,
            MAP_MAP002_29 = 11720,
            MAP_MAP002_32 = 11721,
            MAP_MAP002_33 = 11785,
            MAP_MAP002_36 = 11786,
            MAP_MAP002_37 = 11850,
            MAP_MAP002_40 = 11851,
            MAP_MAP002_41 = 11915,
            MAP_MAP002_44 = 11916,
            MAP_MAP002_45 = 11980,
            MAP_MAP002_54 = 11981,
            MAP_MAP002_55 = 12045,
            MAP_MAP002_58 = 12071,
            MAP_MAP002_59 = 12135,
            MAP_MAP002_62 = 12136,
            MAP_MAP002_63 = 12200,
            MAP_MAP002_66 = 12201,
            MAP_MAP002_67 = 12265,
            MAP_MAP002_70 = 12266,
            MAP_MAP002_71 = 12330,
            MAP_MAP002_74 = 12331,
            MAP_MAP002_75 = 12395,
            MAP_MAP002_78 = 12396,
            MAP_MAP002_79 = 12460,
            MAP_MAP002_8 = 11306,
            MAP_MAP002_82 = 12461,
            MAP_MAP002_83 = 12525,
            MAP_MAP002_86 = 12526,
            MAP_MAP002_87 = 12590,
            MAP_MAP002_9 = 11370,
            MAP_MAP002_90 = 12591,
            MAP_MAP002_91 = 12655,
            MAP_MAP002_GNS = 12656,
            MAP_MAP003_12 = 12743,
            MAP_MAP003_13 = 12807,
            MAP_MAP003_16 = 12808,
            MAP_MAP003_17 = 12872,
            MAP_MAP003_20 = 12873,
            MAP_MAP003_21 = 12937,
            MAP_MAP003_8 = 12658,
            MAP_MAP003_9 = 12722,
            MAP_MAP003_GNS = 12938,
            MAP_MAP004_13 = 13026,
            MAP_MAP004_18 = 13047,
            MAP_MAP004_23 = 13069,
            MAP_MAP004_24 = 13133,
            MAP_MAP004_29 = 13153,
            MAP_MAP004_30 = 13217,
            MAP_MAP004_35 = 13237,
            MAP_MAP004_36 = 13301,
            MAP_MAP004_41 = 13321,
            MAP_MAP004_42 = 13385,
            MAP_MAP004_47 = 13404,
            MAP_MAP004_48 = 13468,
            MAP_MAP004_53 = 13487,
            MAP_MAP004_54 = 13551,
            MAP_MAP004_7 = 12939,
            MAP_MAP004_8 = 13003,
            MAP_MAP004_GNS = 13570,
            MAP_MAP005_10 = 13654,
            MAP_MAP005_11 = 13718,
            MAP_MAP005_13 = 13719,
            MAP_MAP005_14 = 13783,
            MAP_MAP005_16 = 13784,
            MAP_MAP005_17 = 13848,
            MAP_MAP005_19 = 13849,
            MAP_MAP005_20 = 13913,
            MAP_MAP005_23 = 13914,
            MAP_MAP005_24 = 13978,
            MAP_MAP005_27 = 13979,
            MAP_MAP005_28 = 14043,
            MAP_MAP005_31 = 14044,
            MAP_MAP005_32 = 14108,
            MAP_MAP005_35 = 14109,
            MAP_MAP005_36 = 14173,
            MAP_MAP005_39 = 14174,
            MAP_MAP005_40 = 14238,
            MAP_MAP005_7 = 13571,
            MAP_MAP005_8 = 13635,
            MAP_MAP005_GNS = 14239,
            MAP_MAP006_11 = 14240,
            MAP_MAP006_12 = 14304,
            MAP_MAP006_13 = 14330,
            MAP_MAP006_15 = 14331,
            MAP_MAP006_16 = 14395,
            MAP_MAP006_20 = 14396,
            MAP_MAP006_21 = 14460,
            MAP_MAP006_25 = 14461,
            MAP_MAP006_26 = 14525,
            MAP_MAP006_31 = 14526,
            MAP_MAP006_35 = 14544,
            MAP_MAP006_36 = 14608,
            MAP_MAP006_42 = 14613,
            MAP_MAP006_43 = 14677,
            MAP_MAP006_49 = 14682,
            MAP_MAP006_50 = 14746,
            MAP_MAP006_GNS = 14751,
            MAP_MAP007_11 = 14835,
            MAP_MAP007_12 = 14899,
            MAP_MAP007_15 = 14900,
            MAP_MAP007_16 = 14964,
            MAP_MAP007_19 = 14965,
            MAP_MAP007_20 = 15029,
            MAP_MAP007_8 = 14752,
            MAP_MAP007_9 = 14816,
            MAP_MAP007_GNS = 15030,
            MAP_MAP008_11 = 15119,
            MAP_MAP008_12 = 15183,
            MAP_MAP008_15 = 15184,
            MAP_MAP008_16 = 15248,
            MAP_MAP008_19 = 15249,
            MAP_MAP008_20 = 15313,
            MAP_MAP008_27 = 15314,
            MAP_MAP008_28 = 15378,
            MAP_MAP008_31 = 15400,
            MAP_MAP008_32 = 15464,
            MAP_MAP008_35 = 15465,
            MAP_MAP008_36 = 15529,
            MAP_MAP008_39 = 15530,
            MAP_MAP008_40 = 15594,
            MAP_MAP008_7 = 15031,
            MAP_MAP008_8 = 15095,
            MAP_MAP008_GNS = 15595,
            MAP_MAP009_13 = 15677,
            MAP_MAP009_14 = 15741,
            MAP_MAP009_18 = 15742,
            MAP_MAP009_19 = 15806,
            MAP_MAP009_23 = 15807,
            MAP_MAP009_24 = 15871,
            MAP_MAP009_28 = 15872,
            MAP_MAP009_29 = 15936,
            MAP_MAP009_34 = 15937,
            MAP_MAP009_35 = 16001,
            MAP_MAP009_40 = 16002,
            MAP_MAP009_41 = 16066,
            MAP_MAP009_46 = 16067,
            MAP_MAP009_47 = 16131,
            MAP_MAP009_52 = 16132,
            MAP_MAP009_53 = 16196,
            MAP_MAP009_58 = 16197,
            MAP_MAP009_59 = 16261,
            MAP_MAP009_8 = 15596,
            MAP_MAP009_9 = 15660,
            MAP_MAP009_GNS = 16262,
            MAP_MAP010_8 = 16263,
            MAP_MAP010_9 = 16327,
            MAP_MAP010_GNS = 16347,
            MAP_MAP011_11 = 16437,
            MAP_MAP011_12 = 16501,
            MAP_MAP011_14 = 16502,
            MAP_MAP011_17 = 16503,
            MAP_MAP011_18 = 16567,
            MAP_MAP011_22 = 16568,
            MAP_MAP011_27 = 16591,
            MAP_MAP011_28 = 16655,
            MAP_MAP011_33 = 16678,
            MAP_MAP011_39 = 16701,
            MAP_MAP011_40 = 16765,
            MAP_MAP011_43 = 16788,
            MAP_MAP011_47 = 16807,
            MAP_MAP011_49 = 16828,
            MAP_MAP011_53 = 16831,
            MAP_MAP011_7 = 16348,
            MAP_MAP011_8 = 16412,
            MAP_MAP011_9 = 16436,
            MAP_MAP011_GNS = 16852,
            MAP_MAP012_10 = 16935,
            MAP_MAP012_14 = 16938,
            MAP_MAP012_15 = 17002,
            MAP_MAP012_19 = 17005,
            MAP_MAP012_20 = 17069,
            MAP_MAP012_24 = 17072,
            MAP_MAP012_25 = 17136,
            MAP_MAP012_26 = 17139,
            MAP_MAP012_30 = 17142,
            MAP_MAP012_31 = 17206,
            MAP_MAP012_35 = 17209,
            MAP_MAP012_36 = 17273,
            MAP_MAP012_40 = 17276,
            MAP_MAP012_41 = 17340,
            MAP_MAP012_8 = 16853,
            MAP_MAP012_9 = 16917,
            MAP_MAP012_GNS = 17343,
            MAP_MAP013_11 = 17432,
            MAP_MAP013_12 = 17496,
            MAP_MAP013_15 = 17497,
            MAP_MAP013_16 = 17561,
            MAP_MAP013_19 = 17562,
            MAP_MAP013_20 = 17626,
            MAP_MAP013_8 = 17344,
            MAP_MAP013_9 = 17408,
            MAP_MAP013_GNS = 17627,
            MAP_MAP014_12 = 17712,
            MAP_MAP014_13 = 17776,
            MAP_MAP014_16 = 17777,
            MAP_MAP014_17 = 17841,
            MAP_MAP014_20 = 17842,
            MAP_MAP014_21 = 17906,
            MAP_MAP014_25 = 17907,
            MAP_MAP014_26 = 17971,
            MAP_MAP014_30 = 17974,
            MAP_MAP014_31 = 18038,
            MAP_MAP014_35 = 18041,
            MAP_MAP014_36 = 18105,
            MAP_MAP014_40 = 18108,
            MAP_MAP014_41 = 18172,
            MAP_MAP014_7 = 17628,
            MAP_MAP014_8 = 17692,
            MAP_MAP014_9 = 17709,
            MAP_MAP014_GNS = 18175,
            MAP_MAP015_11 = 18258,
            MAP_MAP015_12 = 18322,
            MAP_MAP015_14 = 18323,
            MAP_MAP015_15 = 18387,
            MAP_MAP015_17 = 18388,
            MAP_MAP015_18 = 18452,
            MAP_MAP015_20 = 18453,
            MAP_MAP015_21 = 18517,
            MAP_MAP015_23 = 18518,
            MAP_MAP015_24 = 18582,
            MAP_MAP015_26 = 18583,
            MAP_MAP015_27 = 18647,
            MAP_MAP015_29 = 18648,
            MAP_MAP015_30 = 18712,
            MAP_MAP015_32 = 18713,
            MAP_MAP015_33 = 18777,
            MAP_MAP015_35 = 18778,
            MAP_MAP015_36 = 18842,
            MAP_MAP015_45 = 18843,
            MAP_MAP015_46 = 18907,
            MAP_MAP015_48 = 18925,
            MAP_MAP015_49 = 18989,
            MAP_MAP015_51 = 18990,
            MAP_MAP015_52 = 19054,
            MAP_MAP015_54 = 19055,
            MAP_MAP015_55 = 19119,
            MAP_MAP015_57 = 19120,
            MAP_MAP015_58 = 19184,
            MAP_MAP015_60 = 19185,
            MAP_MAP015_61 = 19249,
            MAP_MAP015_63 = 19250,
            MAP_MAP015_64 = 19314,
            MAP_MAP015_66 = 19315,
            MAP_MAP015_67 = 19379,
            MAP_MAP015_69 = 19380,
            MAP_MAP015_70 = 19444,
            MAP_MAP015_72 = 19445,
            MAP_MAP015_73 = 19509,
            MAP_MAP015_8 = 18176,
            MAP_MAP015_9 = 18240,
            MAP_MAP015_GNS = 19510,
            MAP_MAP016_12 = 19598,
            MAP_MAP016_13 = 19662,
            MAP_MAP016_16 = 19663,
            MAP_MAP016_17 = 19727,
            MAP_MAP016_20 = 19728,
            MAP_MAP016_21 = 19792,
            MAP_MAP016_30 = 19793,
            MAP_MAP016_31 = 19857,
            MAP_MAP016_34 = 19880,
            MAP_MAP016_35 = 19944,
            MAP_MAP016_38 = 19945,
            MAP_MAP016_39 = 20009,
            MAP_MAP016_42 = 20010,
            MAP_MAP016_43 = 20074,
            MAP_MAP016_8 = 19511,
            MAP_MAP016_9 = 19575,
            MAP_MAP016_GNS = 20075,
            MAP_MAP017_10 = 20140,
            MAP_MAP017_9 = 20076,
            MAP_MAP017_GNS = 20162,
            MAP_MAP018_13 = 20253,
            MAP_MAP018_14 = 20317,
            MAP_MAP018_18 = 20320,
            MAP_MAP018_19 = 20384,
            MAP_MAP018_23 = 20387,
            MAP_MAP018_24 = 20451,
            MAP_MAP018_33 = 20454,
            MAP_MAP018_34 = 20518,
            MAP_MAP018_38 = 20544,
            MAP_MAP018_39 = 20608,
            MAP_MAP018_43 = 20611,
            MAP_MAP018_44 = 20675,
            MAP_MAP018_48 = 20678,
            MAP_MAP018_49 = 20742,
            MAP_MAP018_8 = 20163,
            MAP_MAP018_9 = 20227,
            MAP_MAP018_GNS = 20745,
            MAP_MAP019_11 = 20826,
            MAP_MAP019_12 = 20890,
            MAP_MAP019_15 = 20891,
            MAP_MAP019_16 = 20955,
            MAP_MAP019_19 = 20956,
            MAP_MAP019_20 = 21020,
            MAP_MAP019_23 = 21021,
            MAP_MAP019_24 = 21085,
            MAP_MAP019_27 = 21086,
            MAP_MAP019_28 = 21150,
            MAP_MAP019_31 = 21151,
            MAP_MAP019_32 = 21215,
            MAP_MAP019_35 = 21216,
            MAP_MAP019_36 = 21280,
            MAP_MAP019_39 = 21281,
            MAP_MAP019_40 = 21345,
            MAP_MAP019_43 = 21346,
            MAP_MAP019_44 = 21410,
            MAP_MAP019_7 = 20746,
            MAP_MAP019_8 = 20810,
            MAP_MAP019_GNS = 21411,
            MAP_MAP020_11 = 21497,
            MAP_MAP020_12 = 21561,
            MAP_MAP020_15 = 21562,
            MAP_MAP020_16 = 21626,
            MAP_MAP020_19 = 21627,
            MAP_MAP020_20 = 21691,
            MAP_MAP020_7 = 21412,
            MAP_MAP020_8 = 21476,
            MAP_MAP020_GNS = 21692,
            MAP_MAP021_15 = 21781,
            MAP_MAP021_18 = 21805,
            MAP_MAP021_19 = 21869,
            MAP_MAP021_22 = 21870,
            MAP_MAP021_23 = 21934,
            MAP_MAP021_26 = 21935,
            MAP_MAP021_27 = 21999,
            MAP_MAP021_33 = 22000,
            MAP_MAP021_38 = 22021,
            MAP_MAP021_39 = 22085,
            MAP_MAP021_44 = 22104,
            MAP_MAP021_45 = 22168,
            MAP_MAP021_50 = 22187,
            MAP_MAP021_51 = 22251,
            MAP_MAP021_7 = 21693,
            MAP_MAP021_8 = 21757,
            MAP_MAP021_GNS = 22270,
            MAP_MAP022_12 = 22353,
            MAP_MAP022_13 = 22417,
            MAP_MAP022_16 = 22418,
            MAP_MAP022_17 = 22482,
            MAP_MAP022_20 = 22483,
            MAP_MAP022_21 = 22547,
            MAP_MAP022_24 = 22548,
            MAP_MAP022_25 = 22612,
            MAP_MAP022_30 = 22613,
            MAP_MAP022_31 = 22677,
            MAP_MAP022_36 = 22678,
            MAP_MAP022_37 = 22742,
            MAP_MAP022_42 = 22743,
            MAP_MAP022_43 = 22807,
            MAP_MAP022_48 = 22808,
            MAP_MAP022_49 = 22872,
            MAP_MAP022_54 = 22873,
            MAP_MAP022_55 = 22937,
            MAP_MAP022_8 = 22271,
            MAP_MAP022_9 = 22335,
            MAP_MAP022_GNS = 22938,
            MAP_MAP023_10 = 23021,
            MAP_MAP023_12 = 23022,
            MAP_MAP023_13 = 23086,
            MAP_MAP023_15 = 23087,
            MAP_MAP023_16 = 23151,
            MAP_MAP023_19 = 23152,
            MAP_MAP023_20 = 23216,
            MAP_MAP023_24 = 23217,
            MAP_MAP023_25 = 23281,
            MAP_MAP023_8 = 22939,
            MAP_MAP023_9 = 23003,
            MAP_MAP023_GNS = 23282,
            MAP_MAP024_10 = 23362,
            MAP_MAP024_11 = 23426,
            MAP_MAP024_14 = 23427,
            MAP_MAP024_15 = 23491,
            MAP_MAP024_18 = 23492,
            MAP_MAP024_19 = 23556,
            MAP_MAP024_6 = 23283,
            MAP_MAP024_7 = 23347,
            MAP_MAP024_GNS = 23557,
            MAP_MAP025_12 = 23639,
            MAP_MAP025_13 = 23703,
            MAP_MAP025_16 = 23704,
            MAP_MAP025_17 = 23768,
            MAP_MAP025_20 = 23769,
            MAP_MAP025_21 = 23833,
            MAP_MAP025_24 = 23834,
            MAP_MAP025_25 = 23898,
            MAP_MAP025_7 = 23558,
            MAP_MAP025_8 = 23622,
            MAP_MAP025_9 = 23638,
            MAP_MAP025_GNS = 23899,
            MAP_MAP026_8 = 23900,
            MAP_MAP026_9 = 23964,
            MAP_MAP026_GNS = 23988,
            MAP_MAP027_10 = 24070,
            MAP_MAP027_13 = 24071,
            MAP_MAP027_14 = 24135,
            MAP_MAP027_17 = 24136,
            MAP_MAP027_18 = 24200,
            MAP_MAP027_21 = 24201,
            MAP_MAP027_22 = 24265,
            MAP_MAP027_7 = 23989,
            MAP_MAP027_8 = 24053,
            MAP_MAP027_GNS = 24266,
            MAP_MAP028_10 = 24349,
            MAP_MAP028_11 = 24413,
            MAP_MAP028_13 = 24414,
            MAP_MAP028_14 = 24478,
            MAP_MAP028_16 = 24479,
            MAP_MAP028_17 = 24543,
            MAP_MAP028_6 = 24267,
            MAP_MAP028_7 = 24331,
            MAP_MAP028_8 = 24348,
            MAP_MAP028_GNS = 24544,
            MAP_MAP029_10 = 24626,
            MAP_MAP029_13 = 24627,
            MAP_MAP029_14 = 24691,
            MAP_MAP029_17 = 24692,
            MAP_MAP029_18 = 24756,
            MAP_MAP029_21 = 24757,
            MAP_MAP029_22 = 24821,
            MAP_MAP029_7 = 24545,
            MAP_MAP029_8 = 24609,
            MAP_MAP029_GNS = 24822,
            MAP_MAP030_11 = 24904,
            MAP_MAP030_12 = 24968,
            MAP_MAP030_15 = 24969,
            MAP_MAP030_16 = 25033,
            MAP_MAP030_19 = 25034,
            MAP_MAP030_20 = 25098,
            MAP_MAP030_6 = 24823,
            MAP_MAP030_7 = 24887,
            MAP_MAP030_8 = 24903,
            MAP_MAP030_GNS = 25099,
            MAP_MAP031_11 = 25179,
            MAP_MAP031_12 = 25243,
            MAP_MAP031_15 = 25244,
            MAP_MAP031_16 = 25308,
            MAP_MAP031_19 = 25309,
            MAP_MAP031_20 = 25373,
            MAP_MAP031_23 = 25374,
            MAP_MAP031_24 = 25438,
            MAP_MAP031_27 = 25439,
            MAP_MAP031_28 = 25503,
            MAP_MAP031_31 = 25504,
            MAP_MAP031_32 = 25568,
            MAP_MAP031_35 = 25569,
            MAP_MAP031_36 = 25633,
            MAP_MAP031_39 = 25634,
            MAP_MAP031_40 = 25698,
            MAP_MAP031_43 = 25699,
            MAP_MAP031_44 = 25763,
            MAP_MAP031_7 = 25100,
            MAP_MAP031_8 = 25164,
            MAP_MAP031_GNS = 25764,
            MAP_MAP032_11 = 25846,
            MAP_MAP032_14 = 25847,
            MAP_MAP032_15 = 25911,
            MAP_MAP032_18 = 25912,
            MAP_MAP032_19 = 25976,
            MAP_MAP032_22 = 25977,
            MAP_MAP032_23 = 26041,
            MAP_MAP032_8 = 25765,
            MAP_MAP032_9 = 25829,
            MAP_MAP032_GNS = 26042,
            MAP_MAP033_18 = 26136,
            MAP_MAP033_19 = 26200,
            MAP_MAP033_8 = 26043,
            MAP_MAP033_9 = 26107,
            MAP_MAP033_GNS = 26229,
            MAP_MAP034_11 = 26312,
            MAP_MAP034_15 = 26329,
            MAP_MAP034_19 = 26346,
            MAP_MAP034_6 = 26230,
            MAP_MAP034_7 = 26294,
            MAP_MAP034_GNS = 26362,
            MAP_MAP035_11 = 26443,
            MAP_MAP035_12 = 26507,
            MAP_MAP035_15 = 26508,
            MAP_MAP035_16 = 26572,
            MAP_MAP035_19 = 26573,
            MAP_MAP035_20 = 26637,
            MAP_MAP035_23 = 26638,
            MAP_MAP035_24 = 26702,
            MAP_MAP035_27 = 26703,
            MAP_MAP035_28 = 26767,
            MAP_MAP035_31 = 26768,
            MAP_MAP035_32 = 26832,
            MAP_MAP035_35 = 26833,
            MAP_MAP035_36 = 26897,
            MAP_MAP035_39 = 26898,
            MAP_MAP035_40 = 26962,
            MAP_MAP035_43 = 26963,
            MAP_MAP035_44 = 27027,
            MAP_MAP035_7 = 26363,
            MAP_MAP035_8 = 26427,
            MAP_MAP035_GNS = 27028,
            MAP_MAP036_1 = 27029,
            MAP_MAP036_12 = 27160,
            MAP_MAP036_13 = 27224,
            MAP_MAP036_16 = 27225,
            MAP_MAP036_17 = 27289,
            MAP_MAP036_20 = 27290,
            MAP_MAP036_21 = 27354,
            MAP_MAP036_24 = 27355,
            MAP_MAP036_25 = 27419,
            MAP_MAP036_28 = 27420,
            MAP_MAP036_29 = 27484,
            MAP_MAP036_32 = 27485,
            MAP_MAP036_33 = 27549,
            MAP_MAP036_4 = 27030,
            MAP_MAP036_45 = 27550,
            MAP_MAP036_46 = 27614,
            MAP_MAP036_5 = 27094,
            MAP_MAP036_8 = 27095,
            MAP_MAP036_9 = 27159,
            MAP_MAP036_GNS = 27643,
            MAP_MAP037_10 = 27792,
            MAP_MAP037_6 = 27644,
            MAP_MAP037_7 = 27708,
            MAP_MAP037_9 = 27728,
            MAP_MAP037_GNS = 27793,
            MAP_MAP038_11 = 27794,
            MAP_MAP038_12 = 27858,
            MAP_MAP038_15 = 27882,
            MAP_MAP038_16 = 27946,
            MAP_MAP038_19 = 27947,
            MAP_MAP038_20 = 28011,
            MAP_MAP038_23 = 28012,
            MAP_MAP038_24 = 28076,
            MAP_MAP038_27 = 28077,
            MAP_MAP038_28 = 28141,
            MAP_MAP038_31 = 28142,
            MAP_MAP038_32 = 28206,
            MAP_MAP038_35 = 28207,
            MAP_MAP038_36 = 28271,
            MAP_MAP038_39 = 28272,
            MAP_MAP038_40 = 28336,
            MAP_MAP038_43 = 28337,
            MAP_MAP038_44 = 28401,
            MAP_MAP038_47 = 28402,
            MAP_MAP038_48 = 28466,
            MAP_MAP038_GNS = 28467,
            MAP_MAP039_8 = 28468,
            MAP_MAP039_9 = 28532,
            MAP_MAP039_GNS = 28555,
            MAP_MAP040_10 = 28620,
            MAP_MAP040_11 = 28644,
            MAP_MAP040_13 = 28645,
            MAP_MAP040_14 = 28709,
            MAP_MAP040_16 = 28710,
            MAP_MAP040_17 = 28774,
            MAP_MAP040_19 = 28775,
            MAP_MAP040_20 = 28839,
            MAP_MAP040_22 = 28840,
            MAP_MAP040_23 = 28904,
            MAP_MAP040_25 = 28905,
            MAP_MAP040_26 = 28969,
            MAP_MAP040_28 = 28970,
            MAP_MAP040_29 = 29034,
            MAP_MAP040_31 = 29035,
            MAP_MAP040_32 = 29099,
            MAP_MAP040_34 = 29100,
            MAP_MAP040_35 = 29164,
            MAP_MAP040_9 = 28556,
            MAP_MAP040_GNS = 29165,
            MAP_MAP041_11 = 29197,
            MAP_MAP041_19 = 29228,
            MAP_MAP041_20 = 29292,
            MAP_MAP041_5 = 29166,
            MAP_MAP041_GNS = 29311,
            MAP_MAP042_12 = 29393,
            MAP_MAP042_13 = 29457,
            MAP_MAP042_16 = 29458,
            MAP_MAP042_17 = 29522,
            MAP_MAP042_20 = 29523,
            MAP_MAP042_21 = 29587,
            MAP_MAP042_24 = 29588,
            MAP_MAP042_25 = 29652,
            MAP_MAP042_8 = 29312,
            MAP_MAP042_9 = 29376,
            MAP_MAP042_GNS = 29653,
            MAP_MAP043_12 = 29742,
            MAP_MAP043_13 = 29806,
            MAP_MAP043_8 = 29654,
            MAP_MAP043_9 = 29718,
            MAP_MAP043_GNS = 29807,
            MAP_MAP044_11 = 29888,
            MAP_MAP044_12 = 29952,
            MAP_MAP044_15 = 29953,
            MAP_MAP044_16 = 30017,
            MAP_MAP044_19 = 30018,
            MAP_MAP044_20 = 30082,
            MAP_MAP044_23 = 30083,
            MAP_MAP044_24 = 30147,
            MAP_MAP044_27 = 30148,
            MAP_MAP044_28 = 30212,
            MAP_MAP044_31 = 30213,
            MAP_MAP044_32 = 30277,
            MAP_MAP044_35 = 30278,
            MAP_MAP044_36 = 30342,
            MAP_MAP044_39 = 30343,
            MAP_MAP044_40 = 30407,
            MAP_MAP044_43 = 30408,
            MAP_MAP044_44 = 30472,
            MAP_MAP044_7 = 29808,
            MAP_MAP044_8 = 29872,
            MAP_MAP044_GNS = 30473,
            MAP_MAP045_10 = 30557,
            MAP_MAP045_11 = 30621,
            MAP_MAP045_6 = 30474,
            MAP_MAP045_7 = 30538,
            MAP_MAP045_GNS = 30622,
            MAP_MAP046_12 = 30706,
            MAP_MAP046_13 = 30770,
            MAP_MAP046_16 = 30771,
            MAP_MAP046_17 = 30835,
            MAP_MAP046_20 = 30836,
            MAP_MAP046_21 = 30900,
            MAP_MAP046_24 = 30901,
            MAP_MAP046_25 = 30965,
            MAP_MAP046_8 = 30623,
            MAP_MAP046_9 = 30687,
            MAP_MAP046_GNS = 30966,
            MAP_MAP047_16 = 30967,
            MAP_MAP047_17 = 31031,
            MAP_MAP047_20 = 31056,
            MAP_MAP047_21 = 31120,
            MAP_MAP047_25 = 31121,
            MAP_MAP047_26 = 31185,
            MAP_MAP047_30 = 31193,
            MAP_MAP047_31 = 31257,
            MAP_MAP047_35 = 31265,
            MAP_MAP047_36 = 31329,
            MAP_MAP047_40 = 31337,
            MAP_MAP047_41 = 31401,
            MAP_MAP047_45 = 31409,
            MAP_MAP047_46 = 31473,
            MAP_MAP047_50 = 31481,
            MAP_MAP047_51 = 31545,
            MAP_MAP047_55 = 31553,
            MAP_MAP047_56 = 31617,
            MAP_MAP047_60 = 31625,
            MAP_MAP047_61 = 31689,
            MAP_MAP047_GNS = 31697,
            MAP_MAP048_12 = 31780,
            MAP_MAP048_13 = 31844,
            MAP_MAP048_16 = 31845,
            MAP_MAP048_17 = 31909,
            MAP_MAP048_20 = 31910,
            MAP_MAP048_21 = 31974,
            MAP_MAP048_24 = 31975,
            MAP_MAP048_25 = 32039,
            MAP_MAP048_28 = 32040,
            MAP_MAP048_29 = 32104,
            MAP_MAP048_32 = 32105,
            MAP_MAP048_33 = 32169,
            MAP_MAP048_36 = 32170,
            MAP_MAP048_37 = 32234,
            MAP_MAP048_40 = 32235,
            MAP_MAP048_41 = 32299,
            MAP_MAP048_44 = 32300,
            MAP_MAP048_45 = 32364,
            MAP_MAP048_8 = 31698,
            MAP_MAP048_9 = 31762,
            MAP_MAP048_GNS = 32365,
            MAP_MAP049_10 = 32447,
            MAP_MAP049_11 = 32511,
            MAP_MAP049_14 = 32512,
            MAP_MAP049_15 = 32576,
            MAP_MAP049_18 = 32577,
            MAP_MAP049_19 = 32641,
            MAP_MAP049_22 = 32642,
            MAP_MAP049_23 = 32706,
            MAP_MAP049_26 = 32707,
            MAP_MAP049_27 = 32771,
            MAP_MAP049_30 = 32772,
            MAP_MAP049_31 = 32836,
            MAP_MAP049_34 = 32837,
            MAP_MAP049_35 = 32901,
            MAP_MAP049_38 = 32902,
            MAP_MAP049_39 = 32966,
            MAP_MAP049_42 = 32967,
            MAP_MAP049_43 = 33031,
            MAP_MAP049_7 = 32366,
            MAP_MAP049_8 = 32430,
            MAP_MAP049_GNS = 33032,
            MAP_MAP050_11 = 33116,
            MAP_MAP050_12 = 33180,
            MAP_MAP050_15 = 33181,
            MAP_MAP050_16 = 33245,
            MAP_MAP050_19 = 33246,
            MAP_MAP050_20 = 33310,
            MAP_MAP050_23 = 33311,
            MAP_MAP050_24 = 33375,
            MAP_MAP050_27 = 33376,
            MAP_MAP050_28 = 33440,
            MAP_MAP050_31 = 33441,
            MAP_MAP050_32 = 33505,
            MAP_MAP050_35 = 33506,
            MAP_MAP050_36 = 33570,
            MAP_MAP050_39 = 33571,
            MAP_MAP050_40 = 33635,
            MAP_MAP050_43 = 33636,
            MAP_MAP050_44 = 33700,
            MAP_MAP050_7 = 33033,
            MAP_MAP050_8 = 33097,
            MAP_MAP050_GNS = 33701,
            MAP_MAP051_16 = 33790,
            MAP_MAP051_17 = 33854,
            MAP_MAP051_20 = 33878,
            MAP_MAP051_21 = 33942,
            MAP_MAP051_24 = 33943,
            MAP_MAP051_25 = 34007,
            MAP_MAP051_28 = 34008,
            MAP_MAP051_29 = 34072,
            MAP_MAP051_32 = 34073,
            MAP_MAP051_37 = 34094,
            MAP_MAP051_38 = 34158,
            MAP_MAP051_43 = 34179,
            MAP_MAP051_44 = 34243,
            MAP_MAP051_49 = 34264,
            MAP_MAP051_50 = 34328,
            MAP_MAP051_7 = 33702,
            MAP_MAP051_8 = 33766,
            MAP_MAP051_GNS = 34349,
            MAP_MAP052_10 = 34414,
            MAP_MAP052_9 = 34350,
            MAP_MAP052_GNS = 34440,
            MAP_MAP053_10 = 34511,
            MAP_MAP053_19 = 34529,
            MAP_MAP053_22 = 34554,
            MAP_MAP053_7 = 34441,
            MAP_MAP053_8 = 34505,
            MAP_MAP053_GNS = 34566,
            MAP_MAP054_7 = 34567,
            MAP_MAP054_8 = 34631,
            MAP_MAP054_GNS = 34647,
            MAP_MAP055_12 = 34648,
            MAP_MAP055_13 = 34712,
            MAP_MAP055_14 = 34739,
            MAP_MAP055_15 = 34742,
            MAP_MAP055_GNS = 34745,
            MAP_MAP056_0 = 34746,
            MAP_MAP056_11 = 34877,
            MAP_MAP056_12 = 34941,
            MAP_MAP056_16 = 34942,
            MAP_MAP056_17 = 35006,
            MAP_MAP056_21 = 35007,
            MAP_MAP056_22 = 35071,
            MAP_MAP056_26 = 35072,
            MAP_MAP056_27 = 35136,
            MAP_MAP056_3 = 34747,
            MAP_MAP056_31 = 35137,
            MAP_MAP056_32 = 35201,
            MAP_MAP056_36 = 35202,
            MAP_MAP056_37 = 35266,
            MAP_MAP056_4 = 34811,
            MAP_MAP056_47 = 35267,
            MAP_MAP056_48 = 35331,
            MAP_MAP056_7 = 34812,
            MAP_MAP056_8 = 34876,
            MAP_MAP056_GNS = 35350,
            MAP_MAP057_8 = 35351,
            MAP_MAP057_9 = 35415,
            MAP_MAP057_GNS = 35436,
            MAP_MAP058_8 = 35437,
            MAP_MAP058_9 = 35501,
            MAP_MAP058_GNS = 35519,
            MAP_MAP059_8 = 35520,
            MAP_MAP059_9 = 35584,
            MAP_MAP059_GNS = 35603,
            MAP_MAP060_8 = 35604,
            MAP_MAP060_9 = 35668,
            MAP_MAP060_GNS = 35683,
            MAP_MAP061_10 = 35764,
            MAP_MAP061_7 = 35684,
            MAP_MAP061_8 = 35748,
            MAP_MAP061_9 = 35763,
            MAP_MAP061_GNS = 35765,
            MAP_MAP062_11 = 35857,
            MAP_MAP062_12 = 35921,
            MAP_MAP062_15 = 35922,
            MAP_MAP062_16 = 35986,
            MAP_MAP062_19 = 35987,
            MAP_MAP062_20 = 36051,
            MAP_MAP062_7 = 35766,
            MAP_MAP062_8 = 35830,
            MAP_MAP062_GNS = 36052,
            MAP_MAP063_10 = 36134,
            MAP_MAP063_11 = 36198,
            MAP_MAP063_14 = 36199,
            MAP_MAP063_15 = 36263,
            MAP_MAP063_18 = 36264,
            MAP_MAP063_19 = 36328,
            MAP_MAP063_22 = 36329,
            MAP_MAP063_23 = 36393,
            MAP_MAP063_6 = 36053,
            MAP_MAP063_7 = 36117,
            MAP_MAP063_GNS = 36394,
            MAP_MAP064_10 = 36459,
            MAP_MAP064_14 = 36486,
            MAP_MAP064_19 = 36505,
            MAP_MAP064_9 = 36395,
            MAP_MAP064_GNS = 36530,
            MAP_MAP065_8 = 36531,
            MAP_MAP065_9 = 36595,
            MAP_MAP065_GNS = 36612,
            MAP_MAP066_12 = 36694,
            MAP_MAP066_13 = 36758,
            MAP_MAP066_16 = 36759,
            MAP_MAP066_17 = 36823,
            MAP_MAP066_20 = 36824,
            MAP_MAP066_21 = 36888,
            MAP_MAP066_24 = 36889,
            MAP_MAP066_25 = 36953,
            MAP_MAP066_28 = 36954,
            MAP_MAP066_29 = 37018,
            MAP_MAP066_32 = 37019,
            MAP_MAP066_33 = 37083,
            MAP_MAP066_36 = 37084,
            MAP_MAP066_37 = 37148,
            MAP_MAP066_40 = 37149,
            MAP_MAP066_41 = 37213,
            MAP_MAP066_7 = 36613,
            MAP_MAP066_8 = 36677,
            MAP_MAP066_9 = 36693,
            MAP_MAP066_GNS = 37214,
            MAP_MAP067_12 = 37297,
            MAP_MAP067_13 = 37361,
            MAP_MAP067_16 = 37362,
            MAP_MAP067_17 = 37426,
            MAP_MAP067_20 = 37427,
            MAP_MAP067_21 = 37491,
            MAP_MAP067_24 = 37492,
            MAP_MAP067_25 = 37556,
            MAP_MAP067_28 = 37557,
            MAP_MAP067_29 = 37621,
            MAP_MAP067_32 = 37622,
            MAP_MAP067_33 = 37686,
            MAP_MAP067_36 = 37687,
            MAP_MAP067_37 = 37751,
            MAP_MAP067_40 = 37752,
            MAP_MAP067_41 = 37816,
            MAP_MAP067_7 = 37215,
            MAP_MAP067_8 = 37279,
            MAP_MAP067_9 = 37296,
            MAP_MAP067_GNS = 37817,
            MAP_MAP068_11 = 37907,
            MAP_MAP068_12 = 37971,
            MAP_MAP068_15 = 37972,
            MAP_MAP068_16 = 38036,
            MAP_MAP068_19 = 38037,
            MAP_MAP068_20 = 38101,
            MAP_MAP068_28 = 38102,
            MAP_MAP068_29 = 38166,
            MAP_MAP068_32 = 38191,
            MAP_MAP068_33 = 38255,
            MAP_MAP068_36 = 38256,
            MAP_MAP068_37 = 38320,
            MAP_MAP068_40 = 38321,
            MAP_MAP068_41 = 38385,
            MAP_MAP068_7 = 37818,
            MAP_MAP068_8 = 37882,
            MAP_MAP068_GNS = 38386,
            MAP_MAP069_7 = 38387,
            MAP_MAP069_8 = 38451,
            MAP_MAP069_GNS = 38473,
            MAP_MAP070_10 = 38557,
            MAP_MAP070_11 = 38621,
            MAP_MAP070_6 = 38474,
            MAP_MAP070_7 = 38538,
            MAP_MAP070_GNS = 38622,
            MAP_MAP071_12 = 38703,
            MAP_MAP071_13 = 38767,
            MAP_MAP071_16 = 38768,
            MAP_MAP071_17 = 38832,
            MAP_MAP071_20 = 38833,
            MAP_MAP071_21 = 38897,
            MAP_MAP071_24 = 38898,
            MAP_MAP071_25 = 38962,
            MAP_MAP071_28 = 38963,
            MAP_MAP071_29 = 39027,
            MAP_MAP071_32 = 39028,
            MAP_MAP071_33 = 39092,
            MAP_MAP071_36 = 39093,
            MAP_MAP071_37 = 39157,
            MAP_MAP071_40 = 39158,
            MAP_MAP071_41 = 39222,
            MAP_MAP071_44 = 39223,
            MAP_MAP071_45 = 39287,
            MAP_MAP071_8 = 38623,
            MAP_MAP071_9 = 38687,
            MAP_MAP071_GNS = 39288,
            MAP_MAP072_10 = 39369,
            MAP_MAP072_13 = 39370,
            MAP_MAP072_14 = 39434,
            MAP_MAP072_17 = 39435,
            MAP_MAP072_18 = 39499,
            MAP_MAP072_20 = 39500,
            MAP_MAP072_21 = 39564,
            MAP_MAP072_23 = 39565,
            MAP_MAP072_26 = 39566,
            MAP_MAP072_27 = 39630,
            MAP_MAP072_30 = 39631,
            MAP_MAP072_31 = 39695,
            MAP_MAP072_34 = 39696,
            MAP_MAP072_35 = 39760,
            MAP_MAP072_38 = 39761,
            MAP_MAP072_39 = 39825,
            MAP_MAP072_8 = 39289,
            MAP_MAP072_9 = 39353,
            MAP_MAP072_GNS = 39826,
            MAP_MAP073_10 = 39957,
            MAP_MAP073_11 = 40021,
            MAP_MAP073_2 = 39827,
            MAP_MAP073_23 = 40022,
            MAP_MAP073_24 = 40086,
            MAP_MAP073_3 = 39891,
            MAP_MAP073_6 = 39892,
            MAP_MAP073_7 = 39956,
            MAP_MAP073_GNS = 40120,
            MAP_MAP074_10 = 40203,
            MAP_MAP074_12 = 40204,
            MAP_MAP074_13 = 40268,
            MAP_MAP074_15 = 40269,
            MAP_MAP074_16 = 40333,
            MAP_MAP074_18 = 40334,
            MAP_MAP074_19 = 40398,
            MAP_MAP074_21 = 40399,
            MAP_MAP074_22 = 40463,
            MAP_MAP074_24 = 40464,
            MAP_MAP074_25 = 40528,
            MAP_MAP074_27 = 40529,
            MAP_MAP074_28 = 40593,
            MAP_MAP074_30 = 40594,
            MAP_MAP074_31 = 40658,
            MAP_MAP074_33 = 40659,
            MAP_MAP074_34 = 40723,
            MAP_MAP074_8 = 40121,
            MAP_MAP074_9 = 40185,
            MAP_MAP074_GNS = 40724,
            MAP_MAP075_11 = 40806,
            MAP_MAP075_12 = 40870,
            MAP_MAP075_14 = 40871,
            MAP_MAP075_15 = 40935,
            MAP_MAP075_17 = 40936,
            MAP_MAP075_18 = 41000,
            MAP_MAP075_20 = 41001,
            MAP_MAP075_21 = 41065,
            MAP_MAP075_25 = 41066,
            MAP_MAP075_26 = 41130,
            MAP_MAP075_30 = 41131,
            MAP_MAP075_31 = 41195,
            MAP_MAP075_35 = 41196,
            MAP_MAP075_36 = 41260,
            MAP_MAP075_40 = 41261,
            MAP_MAP075_41 = 41325,
            MAP_MAP075_45 = 41326,
            MAP_MAP075_46 = 41390,
            MAP_MAP075_8 = 40725,
            MAP_MAP075_9 = 40789,
            MAP_MAP075_GNS = 41391,
            MAP_MAP076_11 = 41473,
            MAP_MAP076_14 = 41474,
            MAP_MAP076_15 = 41538,
            MAP_MAP076_18 = 41539,
            MAP_MAP076_19 = 41603,
            MAP_MAP076_21 = 41604,
            MAP_MAP076_24 = 41605,
            MAP_MAP076_25 = 41669,
            MAP_MAP076_28 = 41670,
            MAP_MAP076_29 = 41734,
            MAP_MAP076_32 = 41735,
            MAP_MAP076_33 = 41799,
            MAP_MAP076_36 = 41800,
            MAP_MAP076_37 = 41864,
            MAP_MAP076_6 = 41392,
            MAP_MAP076_7 = 41456,
            MAP_MAP076_9 = 41472,
            MAP_MAP076_GNS = 41865,
            MAP_MAP077_11 = 41947,
            MAP_MAP077_12 = 42011,
            MAP_MAP077_14 = 42012,
            MAP_MAP077_15 = 42076,
            MAP_MAP077_17 = 42077,
            MAP_MAP077_18 = 42141,
            MAP_MAP077_20 = 42142,
            MAP_MAP077_21 = 42206,
            MAP_MAP077_25 = 42207,
            MAP_MAP077_26 = 42271,
            MAP_MAP077_30 = 42272,
            MAP_MAP077_31 = 42336,
            MAP_MAP077_35 = 42337,
            MAP_MAP077_36 = 42401,
            MAP_MAP077_40 = 42402,
            MAP_MAP077_41 = 42466,
            MAP_MAP077_45 = 42467,
            MAP_MAP077_46 = 42531,
            MAP_MAP077_8 = 41866,
            MAP_MAP077_9 = 41930,
            MAP_MAP077_GNS = 42532,
            MAP_MAP078_13 = 42615,
            MAP_MAP078_14 = 42679,
            MAP_MAP078_18 = 42680,
            MAP_MAP078_19 = 42744,
            MAP_MAP078_22 = 42745,
            MAP_MAP078_23 = 42809,
            MAP_MAP078_26 = 42810,
            MAP_MAP078_27 = 42874,
            MAP_MAP078_31 = 42875,
            MAP_MAP078_32 = 42939,
            MAP_MAP078_35 = 42940,
            MAP_MAP078_36 = 43004,
            MAP_MAP078_39 = 43005,
            MAP_MAP078_40 = 43069,
            MAP_MAP078_43 = 43070,
            MAP_MAP078_44 = 43134,
            MAP_MAP078_47 = 43135,
            MAP_MAP078_48 = 43199,
            MAP_MAP078_8 = 42533,
            MAP_MAP078_9 = 42597,
            MAP_MAP078_GNS = 43200,
            MAP_MAP079_10 = 43288,
            MAP_MAP079_11 = 43289,
            MAP_MAP079_12 = 43290,
            MAP_MAP079_13 = 43291,
            MAP_MAP079_14 = 43292,
            MAP_MAP079_15 = 43293,
            MAP_MAP079_16 = 43294,
            MAP_MAP079_6 = 43201,
            MAP_MAP079_7 = 43265,
            MAP_MAP079_8 = 43286,
            MAP_MAP079_9 = 43287,
            MAP_MAP079_GNS = 43295,
            MAP_MAP080_11 = 43381,
            MAP_MAP080_12 = 43445,
            MAP_MAP080_15 = 43446,
            MAP_MAP080_16 = 43510,
            MAP_MAP080_19 = 43511,
            MAP_MAP080_20 = 43575,
            MAP_MAP080_23 = 43576,
            MAP_MAP080_24 = 43640,
            MAP_MAP080_27 = 43641,
            MAP_MAP080_28 = 43705,
            MAP_MAP080_31 = 43706,
            MAP_MAP080_32 = 43770,
            MAP_MAP080_35 = 43771,
            MAP_MAP080_36 = 43835,
            MAP_MAP080_39 = 43836,
            MAP_MAP080_40 = 43900,
            MAP_MAP080_6 = 43296,
            MAP_MAP080_7 = 43360,
            MAP_MAP080_8 = 43380,
            MAP_MAP080_GNS = 43901,
            MAP_MAP081_10 = 43984,
            MAP_MAP081_11 = 44048,
            MAP_MAP081_14 = 44049,
            MAP_MAP081_15 = 44113,
            MAP_MAP081_18 = 44114,
            MAP_MAP081_19 = 44178,
            MAP_MAP081_22 = 44179,
            MAP_MAP081_23 = 44243,
            MAP_MAP081_26 = 44244,
            MAP_MAP081_27 = 44308,
            MAP_MAP081_30 = 44309,
            MAP_MAP081_31 = 44373,
            MAP_MAP081_34 = 44374,
            MAP_MAP081_35 = 44438,
            MAP_MAP081_38 = 44439,
            MAP_MAP081_39 = 44503,
            MAP_MAP081_42 = 44504,
            MAP_MAP081_43 = 44568,
            MAP_MAP081_6 = 43902,
            MAP_MAP081_7 = 43966,
            MAP_MAP081_GNS = 44569,
            MAP_MAP082_11 = 44652,
            MAP_MAP082_14 = 44653,
            MAP_MAP082_15 = 44717,
            MAP_MAP082_18 = 44718,
            MAP_MAP082_19 = 44782,
            MAP_MAP082_21 = 44783,
            MAP_MAP082_24 = 44784,
            MAP_MAP082_25 = 44848,
            MAP_MAP082_28 = 44849,
            MAP_MAP082_29 = 44913,
            MAP_MAP082_32 = 44914,
            MAP_MAP082_33 = 44978,
            MAP_MAP082_36 = 44979,
            MAP_MAP082_37 = 45043,
            MAP_MAP082_6 = 44570,
            MAP_MAP082_7 = 44634,
            MAP_MAP082_9 = 44651,
            MAP_MAP082_GNS = 45044,
            MAP_MAP083_10 = 45124,
            MAP_MAP083_12 = 45134,
            MAP_MAP083_14 = 45135,
            MAP_MAP083_16 = 45136,
            MAP_MAP083_18 = 45137,
            MAP_MAP083_20 = 45138,
            MAP_MAP083_22 = 45139,
            MAP_MAP083_24 = 45140,
            MAP_MAP083_26 = 45141,
            MAP_MAP083_28 = 45142,
            MAP_MAP083_38 = 45143,
            MAP_MAP083_8 = 45045,
            MAP_MAP083_9 = 45109,
            MAP_MAP083_GNS = 45164,
            MAP_MAP084_10 = 45244,
            MAP_MAP084_11 = 45308,
            MAP_MAP084_14 = 45309,
            MAP_MAP084_15 = 45373,
            MAP_MAP084_18 = 45374,
            MAP_MAP084_19 = 45438,
            MAP_MAP084_22 = 45439,
            MAP_MAP084_23 = 45503,
            MAP_MAP084_26 = 45504,
            MAP_MAP084_27 = 45568,
            MAP_MAP084_30 = 45569,
            MAP_MAP084_31 = 45633,
            MAP_MAP084_34 = 45634,
            MAP_MAP084_35 = 45698,
            MAP_MAP084_38 = 45699,
            MAP_MAP084_39 = 45763,
            MAP_MAP084_42 = 45764,
            MAP_MAP084_43 = 45828,
            MAP_MAP084_6 = 45165,
            MAP_MAP084_7 = 45229,
            MAP_MAP084_GNS = 45829,
            MAP_MAP085_10 = 45977,
            MAP_MAP085_12 = 45978,
            MAP_MAP085_13 = 46042,
            MAP_MAP085_15 = 46043,
            MAP_MAP085_16 = 46107,
            MAP_MAP085_18 = 46108,
            MAP_MAP085_19 = 46172,
            MAP_MAP085_21 = 46173,
            MAP_MAP085_22 = 46237,
            MAP_MAP085_24 = 46238,
            MAP_MAP085_25 = 46302,
            MAP_MAP085_27 = 46303,
            MAP_MAP085_28 = 46367,
            MAP_MAP085_30 = 46368,
            MAP_MAP085_31 = 46432,
            MAP_MAP085_33 = 46433,
            MAP_MAP085_34 = 46497,
            MAP_MAP085_6 = 45830,
            MAP_MAP085_7 = 45894,
            MAP_MAP085_9 = 45913,
            MAP_MAP085_GNS = 46498,
            MAP_MAP086_10 = 46646,
            MAP_MAP086_12 = 46647,
            MAP_MAP086_13 = 46711,
            MAP_MAP086_15 = 46712,
            MAP_MAP086_16 = 46776,
            MAP_MAP086_18 = 46777,
            MAP_MAP086_19 = 46841,
            MAP_MAP086_21 = 46842,
            MAP_MAP086_22 = 46906,
            MAP_MAP086_24 = 46907,
            MAP_MAP086_25 = 46971,
            MAP_MAP086_27 = 46972,
            MAP_MAP086_28 = 47036,
            MAP_MAP086_30 = 47037,
            MAP_MAP086_31 = 47101,
            MAP_MAP086_33 = 47102,
            MAP_MAP086_34 = 47166,
            MAP_MAP086_6 = 46499,
            MAP_MAP086_7 = 46563,
            MAP_MAP086_9 = 46582,
            MAP_MAP086_GNS = 47167,
            MAP_MAP087_10 = 47251,
            MAP_MAP087_11 = 47252,
            MAP_MAP087_12 = 47253,
            MAP_MAP087_13 = 47254,
            MAP_MAP087_14 = 47255,
            MAP_MAP087_15 = 47256,
            MAP_MAP087_16 = 47257,
            MAP_MAP087_17 = 47258,
            MAP_MAP087_18 = 47259,
            MAP_MAP087_8 = 47168,
            MAP_MAP087_9 = 47232,
            MAP_MAP087_GNS = 47260,
            MAP_MAP088_11 = 47343,
            MAP_MAP088_12 = 47407,
            MAP_MAP088_14 = 47408,
            MAP_MAP088_15 = 47472,
            MAP_MAP088_17 = 47473,
            MAP_MAP088_18 = 47537,
            MAP_MAP088_20 = 47538,
            MAP_MAP088_21 = 47602,
            MAP_MAP088_23 = 47603,
            MAP_MAP088_24 = 47667,
            MAP_MAP088_26 = 47668,
            MAP_MAP088_27 = 47732,
            MAP_MAP088_29 = 47733,
            MAP_MAP088_30 = 47797,
            MAP_MAP088_32 = 47798,
            MAP_MAP088_33 = 47862,
            MAP_MAP088_35 = 47863,
            MAP_MAP088_36 = 47927,
            MAP_MAP088_8 = 47261,
            MAP_MAP088_9 = 47325,
            MAP_MAP088_GNS = 47928,
            MAP_MAP089_10 = 48010,
            MAP_MAP089_11 = 48074,
            MAP_MAP089_14 = 48075,
            MAP_MAP089_15 = 48139,
            MAP_MAP089_18 = 48140,
            MAP_MAP089_19 = 48204,
            MAP_MAP089_22 = 48205,
            MAP_MAP089_23 = 48269,
            MAP_MAP089_26 = 48270,
            MAP_MAP089_27 = 48334,
            MAP_MAP089_30 = 48335,
            MAP_MAP089_31 = 48399,
            MAP_MAP089_34 = 48400,
            MAP_MAP089_35 = 48464,
            MAP_MAP089_38 = 48465,
            MAP_MAP089_39 = 48529,
            MAP_MAP089_42 = 48530,
            MAP_MAP089_43 = 48594,
            MAP_MAP089_6 = 47929,
            MAP_MAP089_7 = 47993,
            MAP_MAP089_GNS = 48595,
            MAP_MAP090_10 = 48675,
            MAP_MAP090_11 = 48739,
            MAP_MAP090_14 = 48740,
            MAP_MAP090_15 = 48804,
            MAP_MAP090_18 = 48805,
            MAP_MAP090_19 = 48869,
            MAP_MAP090_22 = 48870,
            MAP_MAP090_23 = 48934,
            MAP_MAP090_26 = 48935,
            MAP_MAP090_27 = 48999,
            MAP_MAP090_30 = 49000,
            MAP_MAP090_31 = 49064,
            MAP_MAP090_34 = 49065,
            MAP_MAP090_35 = 49129,
            MAP_MAP090_38 = 49130,
            MAP_MAP090_39 = 49194,
            MAP_MAP090_42 = 49195,
            MAP_MAP090_43 = 49259,
            MAP_MAP090_6 = 48596,
            MAP_MAP090_7 = 48660,
            MAP_MAP090_GNS = 49260,
            MAP_MAP091_10 = 49342,
            MAP_MAP091_13 = 49343,
            MAP_MAP091_14 = 49407,
            MAP_MAP091_17 = 49408,
            MAP_MAP091_18 = 49472,
            MAP_MAP091_21 = 49473,
            MAP_MAP091_22 = 49537,
            MAP_MAP091_8 = 49261,
            MAP_MAP091_9 = 49325,
            MAP_MAP091_GNS = 49538,
            MAP_MAP092_12 = 49629,
            MAP_MAP092_13 = 49693,
            MAP_MAP092_16 = 49694,
            MAP_MAP092_17 = 49758,
            MAP_MAP092_20 = 49759,
            MAP_MAP092_21 = 49823,
            MAP_MAP092_30 = 49824,
            MAP_MAP092_31 = 49888,
            MAP_MAP092_34 = 49913,
            MAP_MAP092_35 = 49977,
            MAP_MAP092_38 = 49978,
            MAP_MAP092_39 = 50042,
            MAP_MAP092_42 = 50043,
            MAP_MAP092_43 = 50107,
            MAP_MAP092_8 = 49539,
            MAP_MAP092_9 = 49603,
            MAP_MAP092_GNS = 50108,
            MAP_MAP093_10 = 50192,
            MAP_MAP093_11 = 50256,
            MAP_MAP093_14 = 50257,
            MAP_MAP093_15 = 50321,
            MAP_MAP093_18 = 50322,
            MAP_MAP093_19 = 50386,
            MAP_MAP093_6 = 50109,
            MAP_MAP093_7 = 50173,
            MAP_MAP093_GNS = 50387,
            MAP_MAP094_16 = 50471,
            MAP_MAP094_17 = 50535,
            MAP_MAP094_7 = 50388,
            MAP_MAP094_8 = 50452,
            MAP_MAP094_GNS = 50554,
            MAP_MAP095_11 = 50643,
            MAP_MAP095_12 = 50707,
            MAP_MAP095_15 = 50708,
            MAP_MAP095_16 = 50772,
            MAP_MAP095_19 = 50773,
            MAP_MAP095_20 = 50837,
            MAP_MAP095_28 = 50838,
            MAP_MAP095_29 = 50902,
            MAP_MAP095_32 = 50925,
            MAP_MAP095_33 = 50989,
            MAP_MAP095_36 = 50990,
            MAP_MAP095_37 = 51054,
            MAP_MAP095_40 = 51055,
            MAP_MAP095_41 = 51119,
            MAP_MAP095_7 = 50555,
            MAP_MAP095_8 = 50619,
            MAP_MAP095_GNS = 51120,
            MAP_MAP096_12 = 51215,
            MAP_MAP096_13 = 51279,
            MAP_MAP096_16 = 51282,
            MAP_MAP096_17 = 51346,
            MAP_MAP096_20 = 51349,
            MAP_MAP096_21 = 51413,
            MAP_MAP096_7 = 51121,
            MAP_MAP096_8 = 51185,
            MAP_MAP096_GNS = 51416,
            MAP_MAP097_11 = 51497,
            MAP_MAP097_12 = 51561,
            MAP_MAP097_15 = 51562,
            MAP_MAP097_16 = 51626,
            MAP_MAP097_18 = 51627,
            MAP_MAP097_19 = 51691,
            MAP_MAP097_21 = 51692,
            MAP_MAP097_22 = 51756,
            MAP_MAP097_25 = 51757,
            MAP_MAP097_26 = 51821,
            MAP_MAP097_29 = 51822,
            MAP_MAP097_30 = 51886,
            MAP_MAP097_33 = 51887,
            MAP_MAP097_34 = 51951,
            MAP_MAP097_37 = 51952,
            MAP_MAP097_38 = 52016,
            MAP_MAP097_41 = 52017,
            MAP_MAP097_42 = 52081,
            MAP_MAP097_7 = 51417,
            MAP_MAP097_8 = 51481,
            MAP_MAP097_GNS = 52082,
            MAP_MAP098_10 = 52164,
            MAP_MAP098_11 = 52228,
            MAP_MAP098_14 = 52229,
            MAP_MAP098_15 = 52293,
            MAP_MAP098_18 = 52294,
            MAP_MAP098_19 = 52358,
            MAP_MAP098_22 = 52359,
            MAP_MAP098_23 = 52423,
            MAP_MAP098_26 = 52424,
            MAP_MAP098_27 = 52488,
            MAP_MAP098_30 = 52489,
            MAP_MAP098_31 = 52553,
            MAP_MAP098_34 = 52554,
            MAP_MAP098_35 = 52618,
            MAP_MAP098_38 = 52619,
            MAP_MAP098_39 = 52683,
            MAP_MAP098_42 = 52684,
            MAP_MAP098_43 = 52748,
            MAP_MAP098_6 = 52083,
            MAP_MAP098_7 = 52147,
            MAP_MAP098_GNS = 52749,
            MAP_MAP099_10 = 52829,
            MAP_MAP099_11 = 52893,
            MAP_MAP099_14 = 52894,
            MAP_MAP099_15 = 52958,
            MAP_MAP099_18 = 52959,
            MAP_MAP099_19 = 53023,
            MAP_MAP099_22 = 53024,
            MAP_MAP099_23 = 53088,
            MAP_MAP099_26 = 53089,
            MAP_MAP099_27 = 53153,
            MAP_MAP099_30 = 53154,
            MAP_MAP099_31 = 53218,
            MAP_MAP099_34 = 53219,
            MAP_MAP099_35 = 53283,
            MAP_MAP099_38 = 53284,
            MAP_MAP099_39 = 53348,
            MAP_MAP099_42 = 53349,
            MAP_MAP099_43 = 53413,
            MAP_MAP099_6 = 52750,
            MAP_MAP099_7 = 52814,
            MAP_MAP099_GNS = 53414,
            MAP_MAP100_7 = 53415,
            MAP_MAP100_8 = 53479,
            MAP_MAP100_GNS = 53502,
            MAP_MAP101_10 = 53567,
            MAP_MAP101_9 = 53503,
            MAP_MAP101_GNS = 53579,
            MAP_MAP102_7 = 53580,
            MAP_MAP102_8 = 53644,
            MAP_MAP102_GNS = 53659,
            MAP_MAP103_10 = 53724,
            MAP_MAP103_11 = 53752,
            MAP_MAP103_13 = 53753,
            MAP_MAP103_14 = 53817,
            MAP_MAP103_16 = 53818,
            MAP_MAP103_17 = 53882,
            MAP_MAP103_19 = 53883,
            MAP_MAP103_20 = 53947,
            MAP_MAP103_23 = 53948,
            MAP_MAP103_24 = 54012,
            MAP_MAP103_27 = 54013,
            MAP_MAP103_28 = 54077,
            MAP_MAP103_31 = 54078,
            MAP_MAP103_32 = 54142,
            MAP_MAP103_35 = 54143,
            MAP_MAP103_36 = 54207,
            MAP_MAP103_39 = 54208,
            MAP_MAP103_40 = 54272,
            MAP_MAP103_9 = 53660,
            MAP_MAP103_GNS = 54273,
            MAP_MAP104_10 = 54356,
            MAP_MAP104_8 = 54274,
            MAP_MAP104_9 = 54338,
            MAP_MAP104_GNS = 54359,
            MAP_MAP105_14 = 54438,
            MAP_MAP105_15 = 54502,
            MAP_MAP105_17 = 54516,
            MAP_MAP105_19 = 54519,
            MAP_MAP105_21 = 54522,
            MAP_MAP105_23 = 54525,
            MAP_MAP105_6 = 54360,
            MAP_MAP105_7 = 54424,
            MAP_MAP105_GNS = 54528,
            MAP_MAP106_11 = 54609,
            MAP_MAP106_13 = 54612,
            MAP_MAP106_15 = 54615,
            MAP_MAP106_17 = 54618,
            MAP_MAP106_6 = 54529,
            MAP_MAP106_7 = 54593,
            MAP_MAP106_9 = 54606,
            MAP_MAP106_GNS = 54621,
            MAP_MAP107_11 = 54704,
            MAP_MAP107_13 = 54707,
            MAP_MAP107_15 = 54710,
            MAP_MAP107_17 = 54713,
            MAP_MAP107_6 = 54622,
            MAP_MAP107_7 = 54686,
            MAP_MAP107_9 = 54701,
            MAP_MAP107_GNS = 54716,
            MAP_MAP108_11 = 54800,
            MAP_MAP108_13 = 54803,
            MAP_MAP108_15 = 54806,
            MAP_MAP108_17 = 54809,
            MAP_MAP108_6 = 54717,
            MAP_MAP108_7 = 54781,
            MAP_MAP108_9 = 54797,
            MAP_MAP108_GNS = 54812,
            MAP_MAP109_11 = 54897,
            MAP_MAP109_13 = 54900,
            MAP_MAP109_15 = 54903,
            MAP_MAP109_17 = 54906,
            MAP_MAP109_6 = 54813,
            MAP_MAP109_7 = 54877,
            MAP_MAP109_9 = 54894,
            MAP_MAP109_GNS = 54909,
            MAP_MAP110_11 = 54992,
            MAP_MAP110_13 = 54995,
            MAP_MAP110_15 = 54998,
            MAP_MAP110_17 = 55001,
            MAP_MAP110_6 = 54910,
            MAP_MAP110_7 = 54974,
            MAP_MAP110_9 = 54989,
            MAP_MAP110_GNS = 55004,
            MAP_MAP111_11 = 55082,
            MAP_MAP111_13 = 55085,
            MAP_MAP111_15 = 55088,
            MAP_MAP111_17 = 55091,
            MAP_MAP111_19 = 55094,
            MAP_MAP111_8 = 55005,
            MAP_MAP111_9 = 55069,
            MAP_MAP111_GNS = 55097,
            MAP_MAP112_10 = 55180,
            MAP_MAP112_12 = 55183,
            MAP_MAP112_14 = 55186,
            MAP_MAP112_16 = 55189,
            MAP_MAP112_6 = 55098,
            MAP_MAP112_7 = 55162,
            MAP_MAP112_8 = 55179,
            MAP_MAP112_GNS = 55192,
            MAP_MAP113_11 = 55274,
            MAP_MAP113_13 = 55277,
            MAP_MAP113_15 = 55280,
            MAP_MAP113_17 = 55283,
            MAP_MAP113_6 = 55193,
            MAP_MAP113_7 = 55257,
            MAP_MAP113_9 = 55271,
            MAP_MAP113_GNS = 55286,
            MAP_MAP114_11 = 55371,
            MAP_MAP114_13 = 55374,
            MAP_MAP114_15 = 55377,
            MAP_MAP114_17 = 55380,
            MAP_MAP114_6 = 55287,
            MAP_MAP114_7 = 55351,
            MAP_MAP114_9 = 55368,
            MAP_MAP114_GNS = 55383,
            MAP_MAP115_10 = 55466,
            MAP_MAP115_11 = 55530,
            MAP_MAP115_14 = 55531,
            MAP_MAP115_15 = 55595,
            MAP_MAP115_18 = 55596,
            MAP_MAP115_19 = 55660,
            MAP_MAP115_22 = 55661,
            MAP_MAP115_23 = 55725,
            MAP_MAP115_26 = 55726,
            MAP_MAP115_27 = 55790,
            MAP_MAP115_30 = 55791,
            MAP_MAP115_31 = 55855,
            MAP_MAP115_34 = 55856,
            MAP_MAP115_35 = 55920,
            MAP_MAP115_38 = 55921,
            MAP_MAP115_39 = 55985,
            MAP_MAP115_42 = 55986,
            MAP_MAP115_43 = 56050,
            MAP_MAP115_6 = 55384,
            MAP_MAP115_7 = 55448,
            MAP_MAP115_GNS = 56051,
            MAP_MAP116_6 = 56052,
            MAP_MAP116_7 = 56116,
            MAP_MAP116_GNS = 56123,
            MAP_MAP117_6 = 56124,
            MAP_MAP117_7 = 56188,
            MAP_MAP117_GNS = 56201,
            MAP_MAP118_6 = 56202,
            MAP_MAP118_7 = 56266,
            MAP_MAP118_GNS = 56279,
            MAP_MAP119_6 = 56280,
            MAP_MAP119_7 = 56344,
            MAP_MAP119_GNS = 56356,
            MAP_MAP125_6 = 56357,
            MAP_MAP125_7 = 56421,
            MAP_MAP125_GNS = 56435,
            MENU_BK_FITR_TIM = 72198,
            MENU_BK_FITR2_TIM = 72231,
            MENU_BK_FITR3_TIM = 72264,
            MENU_BK_HONE_TIM = 72099,
            MENU_BK_HONE2_TIM = 72132,
            MENU_BK_HONE3_TIM = 72165,
            MENU_BK_SHOP_TIM = 72000,
            MENU_BK_SHOP2_TIM = 72033,
            MENU_BK_SHOP3_TIM = 72066,
            MENU_FFTSAVE_DAT = 72319,
            MENU_TUTO1_MES = 72298,
            MENU_TUTO1_SCR = 72297,
            MENU_TUTO2_MES = 72301,
            MENU_TUTO2_SCR = 72300,
            MENU_TUTO3_MES = 72305,
            MENU_TUTO3_SCR = 72304,
            MENU_TUTO4_MES = 72309,
            MENU_TUTO4_SCR = 72308,
            MENU_TUTO5_MES = 72312,
            MENU_TUTO5_SCR = 72311,
            MENU_TUTO6_MES = 72316,
            MENU_TUTO6_SCR = 72315,
            MENU_TUTO7_MES = 72318,
            MENU_TUTO7_SCR = 72317,
            OPEN_ENDING_XA = 193873,
            OPEN_FFTEND_STR = 137480,
            OPEN_FFTOP_STR = 97120,
            OPEN_FFTPR_STR = 151904,
            OPEN_FFTPRE_STR = 122936,
            OPEN_FFTST_STR = 86998,
            OPEN_FFTUNIT_STR = 159424,
            OPEN_OPEN_BIN = 86000,
            OPEN_OPNBK_BIN = 86595,
            OPEN_OPNTEX_BIN = 86109,
            SOUND_ENV_SED = 85000,
            SOUND_MUSIC_00_SMD = 85249,
            SOUND_MUSIC_01_SMD = 85254,
            SOUND_MUSIC_02_SMD = 85259,
            SOUND_MUSIC_03_SMD = 85262,
            SOUND_MUSIC_04_SMD = 85265,
            SOUND_MUSIC_05_SMD = 85268,
            SOUND_MUSIC_06_SMD = 85271,
            SOUND_MUSIC_07_SMD = 85274,
            SOUND_MUSIC_08_SMD = 85279,
            SOUND_MUSIC_09_SMD = 85282,
            SOUND_MUSIC_10_SMD = 85284,
            SOUND_MUSIC_11_SMD = 85287,
            SOUND_MUSIC_12_SMD = 85292,
            SOUND_MUSIC_13_SMD = 85297,
            SOUND_MUSIC_14_SMD = 85301,
            SOUND_MUSIC_15_SMD = 85308,
            SOUND_MUSIC_16_SMD = 85312,
            SOUND_MUSIC_17_SMD = 85314,
            SOUND_MUSIC_18_SMD = 85318,
            SOUND_MUSIC_19_SMD = 85327,
            SOUND_MUSIC_20_SMD = 85335,
            SOUND_MUSIC_21_SMD = 85336,
            SOUND_MUSIC_22_SMD = 85337,
            SOUND_MUSIC_23_SMD = 85338,
            SOUND_MUSIC_24_SMD = 85339,
            SOUND_MUSIC_25_SMD = 85340,
            SOUND_MUSIC_26_SMD = 85342,
            SOUND_MUSIC_27_SMD = 85345,
            SOUND_MUSIC_28_SMD = 85346,
            SOUND_MUSIC_29_SMD = 85348,
            SOUND_MUSIC_30_SMD = 85350,
            SOUND_MUSIC_31_SMD = 85351,
            SOUND_MUSIC_32_SMD = 85353,
            SOUND_MUSIC_33_SMD = 85354,
            SOUND_MUSIC_34_SMD = 85355,
            SOUND_MUSIC_35_SMD = 85357,
            SOUND_MUSIC_36_SMD = 85358,
            SOUND_MUSIC_37_SMD = 85360,
            SOUND_MUSIC_38_SMD = 85365,
            SOUND_MUSIC_39_SMD = 85369,
            SOUND_MUSIC_40_SMD = 85372,
            SOUND_MUSIC_41_SMD = 85373,
            SOUND_MUSIC_42_SMD = 85374,
            SOUND_MUSIC_43_SMD = 85376,
            SOUND_MUSIC_44_SMD = 85377,
            SOUND_MUSIC_45_SMD = 85378,
            SOUND_MUSIC_46_SMD = 85379,
            SOUND_MUSIC_47_SMD = 85380,
            SOUND_MUSIC_48_SMD = 85381,
            SOUND_MUSIC_49_SMD = 85382,
            SOUND_MUSIC_50_SMD = 85383,
            SOUND_MUSIC_51_SMD = 85384,
            SOUND_MUSIC_52_SMD = 85385,
            SOUND_MUSIC_53_SMD = 85387,
            SOUND_MUSIC_54_SMD = 85389,
            SOUND_MUSIC_55_SMD = 85391,
            SOUND_MUSIC_56_SMD = 85392,
            SOUND_MUSIC_57_SMD = 85394,
            SOUND_MUSIC_58_SMD = 85396,
            SOUND_MUSIC_59_SMD = 85398,
            SOUND_MUSIC_60_SMD = 85399,
            SOUND_MUSIC_61_SMD = 85401,
            SOUND_MUSIC_62_SMD = 85403,
            SOUND_MUSIC_63_SMD = 85406,
            SOUND_MUSIC_64_SMD = 85407,
            SOUND_MUSIC_65_SMD = 85409,
            SOUND_MUSIC_66_SMD = 85413,
            SOUND_MUSIC_67_SMD = 85416,
            SOUND_MUSIC_68_SMD = 85418,
            SOUND_MUSIC_69_SMD = 85420,
            SOUND_MUSIC_70_SMD = 85422,
            SOUND_MUSIC_71_SMD = 85424,
            SOUND_MUSIC_72_SMD = 85427,
            SOUND_MUSIC_73_SMD = 85430,
            SOUND_MUSIC_74_SMD = 85431,
            SOUND_MUSIC_75_SMD = 85433,
            SOUND_MUSIC_76_SMD = 85435,
            SOUND_MUSIC_77_SMD = 85440,
            SOUND_MUSIC_78_SMD = 85445,
            SOUND_MUSIC_79_SMD = 85448,
            SOUND_MUSIC_80_SMD = 85449,
            SOUND_MUSIC_81_SMD = 85452,
            SOUND_MUSIC_82_SMD = 85459,
            SOUND_MUSIC_83_SMD = 85464,
            SOUND_MUSIC_84_SMD = 85467,
            SOUND_MUSIC_85_SMD = 85471,
            SOUND_MUSIC_86_SMD = 85473,
            SOUND_MUSIC_87_SMD = 85476,
            SOUND_MUSIC_88_SMD = 85478,
            SOUND_MUSIC_89_SMD = 85480,
            SOUND_MUSIC_90_SMD = 85482,
            SOUND_MUSIC_91_SMD = 85485,
            SOUND_MUSIC_92_SMD = 85491,
            SOUND_MUSIC_93_SMD = 85492,
            SOUND_MUSIC_94_SMD = 85495,
            SOUND_MUSIC_95_SMD = 85497,
            SOUND_MUSIC_96_SMD = 85502,
            SOUND_MUSIC_97_SMD = 85504,
            SOUND_MUSIC_98_SMD = 85506,
            SOUND_MUSIC_99_SMD = 85508,
            SOUND_SYSTEM_SED = 85002,
            SOUND_WAVESET_WD = 85007,
            WORLD_SNPLBIN_BIN = 76025,
            WORLD_SNPLMES_BIN = 76061,
            WORLD_WLDBK_BIN = 76181,
            WORLD_WLDCORE_BIN = 84041,
            WORLD_WLDMES_BIN = 73561,
            WORLD_WLDPIC_BIN = 73134,
            WORLD_WLDTEX_TM2 = 73000,
            WORLD_WORLD_BIN = 84261,
            BATTLE_BIN = 1000,
            SCEAP_DAT = 198,
            SCUS_942_21 = 24,
            SYSTEM_CNF = 23,
            BATTLE_00_SPR = 230064,
            BATTLE_01_SPR = 230096,
            BATTLE_02_SPR = 230128,
            BATTLE_03_SPR = 230160,
            BATTLE_04_SPR = 230192,
            BATTLE_05_SPR = 230224,
            BATTLE_06_SPR = 230256,
            BATTLE_07_SPR = 230288,
            BATTLE_08_SPR = 230320,
            BATTLE_09_SPR = 230352,
            BATTLE_0A_SPR = 230384,
            BATTLE_0B_SPR = 230416,
            BATTLE_0C_SPR = 230448,
            BATTLE_0D_SPR = 230480,
            BATTLE_0E_SPR = 230512,
            BATTLE_0F_SPR = 230544,
            BATTLE_10_SPR = 230576,
            BATTLE_11_SPR = 230608,
            BATTLE_12_SPR = 230640,
            BATTLE_13_SPR = 230672,
            BATTLE_14_SPR = 230704,
            BATTLE_15_SPR = 230736,
            BATTLE_16_SPR = 230768,
            BATTLE_17_SPR = 230800,
            BATTLE_18_SPR = 230832,
            BATTLE_19_SPR = 230864,
            BATTLE_1A_SPR = 230896,
            BATTLE_1B_SPR = 230928,
            BATTLE_1C_SPR = 230960,
            BATTLE_1D_SPR = 230992,
            BATTLE_1E_SPR = 231024,
            BATTLE_1F_SPR = 231056,
            BATTLE_20_SPR = 231088,
            BATTLE_21_SPR = 231120,
            BATTLE_22_SPR = 231152,
            BATTLE_23_SPR = 231184,
            BATTLE_24_SPR = 231216,
            BATTLE_25_SPR = 231248,
            BATTLE_26_SPR = 231280,
            BATTLE_27_SPR = 231312,
            BATTLE_28_SPR = 231344,
            BATTLE_29_SPR = 231376,
            BATTLE_2A_SPR = 231408,
            BATTLE_2B_SPR = 231440,
            BATTLE_2C_SPR = 231472,
            BATTLE_2D_SPR = 231504,
            BATTLE_2E_SPR = 231536,
            BATTLE_2F_SPR = 231568,
            BATTLE_30_SPR = 231600,
            BATTLE_31_SPR = 231632,
            BATTLE_32_SPR = 231664,
            BATTLE_33_SPR = 231696,
            BATTLE_34_SPR = 231728,
            BATTLE_35_SPR = 231760,
            BATTLE_36_SPR = 231792,
            BATTLE_37_SPR = 231824,
            BATTLE_38_SPR = 231856,
            BATTLE_39_SPR = 231888,
            BATTLE_3A_SPR = 231920,
            BATTLE_3B_SPR = 231952,
            BATTLE_3C_SPR = 231984,
            BATTLE_3D_SPR = 232016,
            BATTLE_3E_SPR = 232048,
            BATTLE_3F_SPR = 232080,
            BATTLE_40_SPR = 232112,
            BATTLE_41_SPR = 232144,
            BATTLE_42_SPR = 232176,
            BATTLE_43_SPR = 232208,
            BATTLE_44_SPR = 232240,
            BATTLE_45_SPR = 232272,
            BATTLE_46_SPR = 232304,
            BATTLE_47_SPR = 232336,
            BATTLE_48_SPR = 232368,
            BATTLE_49_SPR = 232400,
            BATTLE_4A_SPR = 232432,
            BATTLE_4B_SPR = 232464,
            BATTLE_4C_SPR = 232496,
            BATTLE_4D_SPR = 232528,
            BATTLE_4E_SPR = 232560,
            BATTLE_4F_SPR = 232592,
            BATTLE_50_SPR = 232624,
            BATTLE_51_SPR = 232656,
            BATTLE_52_SPR = 232688,
            BATTLE_53_SPR = 232720,
            BATTLE_54_SPR = 232752,
            BATTLE_55_SPR = 232784,
            BATTLE_56_SPR = 232816,
            BATTLE_57_SPR = 232848,
            BATTLE_58_SPR = 232880,
            BATTLE_59_SPR = 232912,
            BATTLE_5A_SPR = 232944,
            BATTLE_5B_SPR = 232976,
            BATTLE_5C_SPR = 233008,
            BATTLE_5D_SPR = 233040,
            BATTLE_5E_SPR = 233072,
            BATTLE_5F_SPR = 233104,
            BATTLE_60_SPR = 233136,
            BATTLE_61_SPR = 233168,
            BATTLE_62_SPR = 233200,
            BATTLE_63_SPR = 233232,
            BATTLE_64_SPR = 233264,
            BATTLE_65_SPR = 233296,
            BATTLE_66_SPR = 233328,
            BATTLE_67_SPR = 233360,
            BATTLE_68_SPR = 233392,
            BATTLE_69_SPR = 233424,
            BATTLE_6A_SPR = 233456,
            BATTLE_6B_SPR = 233488,
            BATTLE_6C_SPR = 233520,
            BATTLE_6D_SPR = 233552,
            BATTLE_6E_SPR = 233584,
            BATTLE_6F_SPR = 233616,
            BATTLE_70_SPR = 233648,
            BATTLE_71_SPR = 233680,
            BATTLE_72_SPR = 233712,
            BATTLE_73_SPR = 233744,
            BATTLE_74_SPR = 233776,
            BATTLE_75_SPR = 233808,
            BATTLE_76_SPR = 233840,
            BATTLE_77_SPR = 233872,
            BATTLE_78_SPR = 233904,
            BATTLE_79_SPR = 233936,
            BATTLE_7A_SPR = 233968,
            BATTLE_7B_SPR = 234000,
            BATTLE_7C_SPR = 234032,
            BATTLE_7D_SPR = 234064,
            BATTLE_7E_SPR = 234096,
            BATTLE_7F_SPR = 234128,
            BATTLE_80_SPR = 234160,
            BATTLE_81_SPR = 234192,
            BATTLE_82_SPR = 234224,
            BATTLE_83_SPR = 234256,
            BATTLE_84_SPR = 234288,
            BATTLE_85_SPR = 234320,
            BATTLE_86_SPR = 234352,
            BATTLE_87_SPR = 234384,
            BATTLE_87_SP2 = 60161,
            BATTLE_88_SPR = 234416,
            BATTLE_88_SP2 = 60209,
            BATTLE_89_SPR = 234448,
            BATTLE_8A_SPR = 234480,
            BATTLE_8B_SPR = 234512,
            BATTLE_8C_SPR = 234544,
            BATTLE_8C_SP2 = 60113,
            BATTLE_8D_SPR = 234576,
            BATTLE_8D_SP2 = 60321,
            BATTLE_8E_SPR = 234608,
            BATTLE_8E_SP2 = 60337,
            BATTLE_8F_SPR = 234640,
            BATTLE_90_SPR = 234672,
            BATTLE_90_SP2 = 60289,
            BATTLE_91_SPR = 234704,
            BATTLE_91_SP2 = 60305,
            BATTLE_92_SPR = 234736,
            BATTLE_92_SP2 = 60129,
            BATTLE_93_SPR = 234768,
            BATTLE_94_SPR = 234800,
            BATTLE_94_SP2 = 60193,
            BATTLE_95_SPR = 234832,
            BATTLE_95_SP2 = 60145,
            BATTLE_96_SPR = 234864,
            BATTLE_97_SPR = 234896,
            BATTLE_98_SPR = 234928,
            BATTLE_98_SP2 = 60177,
            BATTLE_99_SPR = 234960,
            BATTLE_99_2_SP2 = 60273,
            BATTLE_99_3_SP2 = 60257,
            BATTLE_99_4_SP2 = 60225,
            BATTLE_99_5_SP2 = 60241,
        }
        public class KnownPosition : PatcherLib.Iso.KnownPosition
        {
            public Sectors Sector { get; private set; }
            public int StartLocation { get; private set; }
            //public int Length { get; private set; }
            private int length;
            public override int Length
            {
                get { return length; }
            }
            public KnownPosition( Sectors sector, int startLocation, int length )
            {
                Sector = sector;
                StartLocation = startLocation;
                this.length = length;
            }

            public override PatcherLib.Datatypes.PatchedByteArray GetPatchedByteArray( byte[] bytes )
            {
                return new PatcherLib.Datatypes.PatchedByteArray( Sector, StartLocation, bytes );
            }

            public override IList<byte> ReadIso( Stream iso )
            {
                return PsxIso.ReadFile( iso, this );
            }

            public override void PatchIso( Stream iso, IList<byte> bytes )
            {
                PsxIso.PatchPsxIso( iso, GetPatchedByteArray( bytes.ToArray() ) );
            }
        }

        public const int BattleDirectoryEntrySector = 56436;
        public const int BattleDirectoryEntryLength = 6;
        public const int DummyDirectoryEntrySector = 230000;
        public const int DummyDirectoryEntryLength = 1;
    }
}